﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using PlusNameSpace;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
        Program objProgram = new Program() ; 
        
        Console.WriteLine("I. Convert values between C# double and MATLAB MWNumericArray");
        objProgram.TransferScalarValues() ;

        Console.WriteLine("II. Convert values between C# vector and MATLAB MWNumericArray");
        objProgram.TransferVectorValues() ;

        Console.WriteLine("III, Convert values between C# matrix and MATLAB MWNumericArray");
        objProgram.TransferMatrixValues() ;
    }      
    /* ******************* */
    public void TransferScalarValues() 
    {
      //1a. Transfer real value from double to MWNumericArray
      double db_a = 1.1   ;
      MWNumericArray mw_a = new MWNumericArray(db_a) ;
      Console.WriteLine("1a. double to MWNumericArray: mw_a = {0}\n", mw_a) ;

      double db_b = 2.2   ;
      MWNumericArray mw_b = new MWNumericArray(db_b) ;

      // Convert values from MWNumericArray to double
      //1b. Transfer real value from MWNumericArray to double    
      MWNumericArray mw_c = null ;        
      Plusclass objMatlab = new Plusclass() ;
      mw_c = (MWNumericArray) objMatlab.myplus(mw_a, mw_b) ;

      double db_c = mw_c.ToScalarDouble() ;
      Console.WriteLine("1b. MWNumericArray to double: db_c = {0}\n", db_c) ; 

      //1c. Transfer complex value from double to MWNumericArray
      double dbReal_a = 1.1 ;
      double dbImag_a = 2.2 ;

      double dbReal_b = 3.3 ;
      double dbImag_b = 4.4 ;           

      MWNumericArray mwComplex_a = new MWNumericArray(dbReal_a, dbImag_a) ;
      MWNumericArray mwComplex_b = new MWNumericArray(dbReal_b, dbImag_b) ;
      Console.WriteLine("1c. double to MWNumericArray Complex: mwComplex_a = {0}\n", mwComplex_a) ;

      //1d. Transfer complex value from MWNumericArray to double
      MWNumericArray mwComplex_c = null ;
      mwComplex_c = (MWNumericArray) objMatlab.myplus(mwComplex_a, mwComplex_b) ;

      Console.WriteLine(mwComplex_c) ;
      Console.WriteLine();

      double[] dbRealBuffer_c = new double[1] ;
      double[] dbImagBuffer_c = new double[1] ;

      dbRealBuffer_c = (double[]) mwComplex_c.ToVector(MWArrayComponent.Real) ;

      double dbReal_c = dbRealBuffer_c[0] ;
      double dbImag_c = 0 ;
      if (mwComplex_c.IsComplex)
      {
          dbImagBuffer_c = (double[]) mwComplex_c.ToVector(MWArrayComponent.Imaginary) ;
          dbImag_c = dbImagBuffer_c[0] ;
      }

      Console.WriteLine("1d. MWNumericArray Complex to double") ;
      Console.WriteLine("The real value dbReal_c = {0}: ", dbReal_c) ;
      Console.WriteLine("The imag value dbImag_c = {0}: ", dbImag_c) ;
      Console.WriteLine() ;
    }

    /* ******************* */
    public void TransferVectorValues()
    {
      //2a. transfer real value of a vector from double to MWNumericArray
      double[] db_a = { 1.1, 2.2, 3.3 } ;
      double[] db_b = { 4.4, 5.5, 6.6 } ;

      MWNumericArray mw_a = new MWNumericArray(db_a) ;
      MWNumericArray mw_b = new MWNumericArray(db_b) ;
      Console.WriteLine("2a. double to MWNumericArray in a real vector mw_a: \n{0}", mw_a) ;

      //2b. transfer real value of a vector from MWNumericArray to double
      Plusclass objMatlab = new Plusclass() ;

      MWNumericArray mw_c = null ;        
      mw_c = (MWNumericArray) objMatlab.myplus(mw_a, mw_b) ;
      double[] db_c = (double[]) mw_c.ToVector(MWArrayComponent.Real) ;
      Console.WriteLine("2b. MWNumericArray to double in a real vector: \n") ;
      PrintVector(db_c) ;          

      //2c. transfer complex value of a vector from double to MWNumericArray
      double[] dbReal_a = { 1.1, 2.2, 3.3 } ;
      double[] dbImag_a = { 4.4, 5.5, 6.6 } ;

      double[] dbReal_b = { 1.1, 2.2, 3.3 } ;
      double[] dbImag_b = { 4.4, 5.5, 6.6 } ;

      MWNumericArray mwComplex_a = new MWNumericArray(dbReal_a, dbImag_a) ;
      MWNumericArray mwComplex_b = new MWNumericArray(dbReal_b, dbImag_b) ;
      Console.WriteLine("2c. complex vector double to MWNumericArray \n{0}", mwComplex_a) ;

      MWNumericArray mwComplex_c = null ;        
      mwComplex_c = (MWNumericArray) objMatlab.myplus(mwComplex_a, mwComplex_b) ;

      //2d. Transfer complex value of a vector from MWNumericArray to double
      double[] dbReal_c = (double[])mwComplex_c.ToVector(MWArrayComponent.Real) ;
      double[] dbImag_c = (double[])Array.CreateInstance(typeof(double), dbReal_c.Length) ;

      if (mwComplex_c.IsComplex)
      {
          dbImag_c = (double[])mwComplex_c.ToVector(MWArrayComponent.Imaginary) ;
      }

      Console.WriteLine("2d. MWNumericArray to complex vector double") ;
      Console.WriteLine("   The real value of the complex vector :") ;
      PrintVector(dbReal_c) ;

      Console.WriteLine("   The imag value of the complex vector :") ;
      PrintVector(dbImag_c) ;
    }

    /* ******************************** */
    public void TransferMatrixValues()
    {
      //3a. Transfer real value of a matrix from double to MWNumericArray
      double[,] db_A = {{ 1.1, 2.2, 3.3} , {4.4, 5.5, 6.6} ,  {7.7, 8.8, 9.9} } ;
      double[,] db_B = {{ 101, 102, 103} , {104, 105, 160} ,  {107, 108, 109} } ;

      MWNumericArray mw_A = new MWNumericArray(db_A) ;
      MWNumericArray mw_B = new MWNumericArray(db_B) ;
      Console.WriteLine("3a. double to MWNumericArray in a real matrix mw_A: {0}\n", mw_A) ;

      //3b. Transfer real value of a matrix from MWNumericArray to double
      Plusclass objMatlab = new Plusclass() ;
      MWNumericArray mw_C = null ; 
      mw_C = (MWNumericArray) objMatlab.myplus(mw_A, mw_B) ;
      double[,] db_C = (double[,]) mw_C.ToArray(MWArrayComponent.Real) ;

      Console.WriteLine("3b. MWNumericArray to double in a real matrix mw_A: \n") ;
      PrintMatrix(db_C) ;


      //3c. transfer complex value of a matrix from double to MWNumericArray
      double[,] dbReal_A = {{ 1.1, 2.2, 3.3}, {4.4, 5.5, 6.6},  {7.7, 8.8, 9.9} } ;
      double[,] dbImag_A = {{ 11 , 12 , 13 }, {14 , 15 , 16 },  {17 , 18 , 19 } } ;

      double[,] dbReal_B = {{ 10.1, 20.2, 30.3}, {40.4, 50.5, 60.6},  {70.7, 80.8, 90.9} } ;
      double[,] dbImag_B = {{ 101 , 102 , 103 }, {104 , 105 , 106 },  {107 , 108 , 109 } } ;

      MWNumericArray mwComplex_A = new MWNumericArray(dbReal_A, dbImag_A) ;
      MWNumericArray mwComplex_B = new MWNumericArray(dbReal_A, dbImag_B) ;
      Console.WriteLine("3c. double to MWNumericArray in a complex matrix mwComplex_A {0}\n", mwComplex_A) ;

      //3d. Transfer complex value of a matrix from MWNumericArray to double
      MWNumericArray mwComplex_C = null ;
      mwComplex_C = (MWNumericArray) objMatlab.myplus(mwComplex_A, mwComplex_B) ;

      double[,] dbReal_C = (double[,])mwComplex_C.ToArray(MWArrayComponent.Real) ;

      int row = dbReal_C.GetUpperBound(0) - dbReal_C.GetLowerBound(0) + 1;
      int col = dbReal_C.GetUpperBound(1) - dbReal_C.GetLowerBound(1) + 1;

      double[,] dbImag_C = (double[,])Array.CreateInstance(typeof(double), row, col) ;

      if (mwComplex_C.IsComplex)
      {
          dbImag_C = (double[,])mwComplex_C.ToArray(MWArrayComponent.Imaginary) ;
      }

      Console.WriteLine("3d. MWNumericArray to double in a complex matrix") ;
      Console.WriteLine("   The real value of the complex matrix :") ;
      PrintMatrix(dbReal_C) ;

      Console.WriteLine("   The imag value of the complex matrix :") ;
      PrintMatrix(dbImag_C) ;
    }

    /* ******************************** */
    public static void PrintVector(double[] vector)
    {
      int i, row;
      row = vector.GetUpperBound(0) - vector.GetLowerBound(0) + 1;

      for (i = 0; i < row; i++)
      {
        Console.Write("{0} \t", vector.GetValue(i).ToString());
        Console.WriteLine();
      }
    }

    /* ******************************** */
    public static void PrintMatrix(double[,] matrix)  
    {    
      int i, j, row, col ;
      row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
      col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++) 
        {                    
          Console.Write("{0} \t", matrix.GetValue(i,j).ToString() ) ;
        }
        Console.WriteLine() ;
      }
    }

/* ******************* */
/* ******************* */
/* ******************* */
    }
}
