﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using CurveFittingsNameSpace ;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Console.WriteLine(" Matlab C# for Curve Fititng Toolbox. ") ; 
      Program objProgram = new Program() ;

      //objProgram.CurveFittingInfo() ;
      //objProgram.CurveFittingPlots() ;
      objProgram.CurveFittingPlotsAdvance() ;
    }

    /* **************************** */
    public void CurveFittingInfo()
    {
      double[]X = {0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600} ;

      double[]Y = {0.4000, 0.2426, 0.1472, 0.0893, 0.0541, 0.0328, 0.0199, 
            0.0121, 0.0073, 0.0044, 0.0027, 0.0016,  0.0010 } ;

      MWNumericArray mw_X = new MWNumericArray(X) ;
      MWNumericArray mw_Y = new MWNumericArray(Y) ;

      MWArray [] mw_ArrayOut              = null ;

      MWCharArray mwChar_formula          = null ;
      MWCellArray mwCell_coeffnames       = null ;
      MWNumericArray mwDouble_coeffvalues = null ;
      MWNumericArray mwDouble_confint     = null ;

      MWStructArray mwStructure_info      = null ;

      String curveFitLib = "exp2" ;
      
      CurveFittings objMatlab = new CurveFittings() ;
      mw_X = (MWNumericArray)objMatlab.mytranspose(mw_X) ;
      mw_Y = (MWNumericArray)objMatlab.mytranspose(mw_Y) ;

      mw_ArrayOut = objMatlab.CurveFitting(5,mw_X, mw_Y, curveFitLib) ;
      
      mwChar_formula       = (MWCharArray) mw_ArrayOut[0] ;
      mwCell_coeffnames    = (MWCellArray) mw_ArrayOut[1] ;
      mwDouble_coeffvalues = (MWNumericArray) mw_ArrayOut[2] ;
      mwDouble_confint     = (MWNumericArray) mw_ArrayOut[3] ;
      mwStructure_info     = (MWStructArray)  mw_ArrayOut[4] ;

      // This output works as well
      /*
      Console.WriteLine(mwChar_formula    ) ;
      Console.WriteLine(mwCell_coeffnames   ) ;
      Console.WriteLine(mwDouble_coeffvalues) ;
      Console.WriteLine(mwDouble_confint  ) ;
      Console.WriteLine(mwStructure_info   ) ;
      */

      String str_formula = mwChar_formula.ToString() ;  
      Console.WriteLine("Curve fitting formula :") ;
      Console.WriteLine(str_formula) ;
      Console.WriteLine() ;

      String str_coeffnames = mwCell_coeffnames.ToString() ;
      Console.WriteLine("Coefficient names:") ;
      Console.WriteLine(str_coeffnames) ;
      Console.WriteLine() ;

      double[] db_coeffvalues = (double[])mwDouble_coeffvalues.ToVector(MWArrayComponent.Real) ;
      Console.WriteLine("Coefficient values:") ;
      PrintVector(db_coeffvalues);
      Console.WriteLine() ;

      double[,] confidentInterval = (double[,])mwDouble_confint.ToArray(MWArrayComponent.Real) ;
      Console.WriteLine("Confidence interval:");
      PrintMatrix(confidentInterval) ;
      Console.WriteLine() ;

      Console.WriteLine("Error info:") ;
      String str_info = mwStructure_info.ToString() ;
      Console.WriteLine(str_info) ;

      // Free memory
      mw_X.Dispose() ;
      mw_Y.Dispose() ;

      MWNumericArray.DisposeArray(mw_ArrayOut) ;

      mwChar_formula.Dispose() ;     
      mwCell_coeffnames.Dispose() ;    
      mwDouble_coeffvalues.Dispose() ;
      mwDouble_confint.Dispose() ;  

      mwStructure_info.Dispose() ;   
    }
    
    /* **************************** */
    public void CurveFittingPlots()
    {
      double[]X = {0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600} ;

      double[]Y = {0.4000, 0.2426, 0.1472, 0.0893, 0.0541, 0.0328, 0.0199, 
            0.0121, 0.0073, 0.0044, 0.0027, 0.0016,  0.0010 } ;

      MWNumericArray mw_X = new MWNumericArray(X) ;
      MWNumericArray mw_Y = new MWNumericArray(Y) ;

      MWArray [] mw_ArrayOut              = null ;

      MWCharArray mwChar_formula          = null ;
      MWCellArray mwCell_coeffnames       = null ;
      MWNumericArray mwDouble_coeffvalues = null ;
      MWNumericArray mwDouble_confint     = null ;

      MWStructArray mwStructure_info      = null ;

      String curveFitLib = "poly3" ;
      
      CurveFittings objMatlab = new CurveFittings() ;
      mw_X = (MWNumericArray)objMatlab.mytranspose(mw_X) ;
      mw_Y = (MWNumericArray)objMatlab.mytranspose(mw_Y) ;

      mw_ArrayOut = objMatlab.CurveFittingWithPlots(5,mw_X, mw_Y, curveFitLib) ;
      
      mwChar_formula       = (MWCharArray) mw_ArrayOut[0] ;
      mwCell_coeffnames    = (MWCellArray) mw_ArrayOut[1] ;
      mwDouble_coeffvalues = (MWNumericArray) mw_ArrayOut[2] ;
      mwDouble_confint     = (MWNumericArray) mw_ArrayOut[3] ;
      mwStructure_info     = (MWStructArray)  mw_ArrayOut[4] ;

      // This output works as well
      /*
      Console.WriteLine(mwChar_formula    ) ;
      Console.WriteLine(mwCell_coeffnames   ) ;
      Console.WriteLine(mwDouble_coeffvalues) ;
      Console.WriteLine(mwDouble_confint  ) ;
      Console.WriteLine(mwStructure_info   ) ;
      */

      String str_formula = mwChar_formula.ToString() ;  
      Console.WriteLine("Curve fitting formula :") ;
      Console.WriteLine(str_formula) ;
      Console.WriteLine() ;

      String str_coeffnames = mwCell_coeffnames.ToString() ;
      Console.WriteLine("Coefficient names:") ;
      Console.WriteLine(str_coeffnames) ;
      Console.WriteLine() ;

      double[] db_coeffvalues = (double[])mwDouble_coeffvalues.ToVector(MWArrayComponent.Real) ;
      Console.WriteLine("Coefficient values:") ;
      PrintVector(db_coeffvalues);
      Console.WriteLine() ;

      double[,] confidentInterval = (double[,])mwDouble_confint.ToArray(MWArrayComponent.Real) ;
      Console.WriteLine("Confidence interval:");
      PrintMatrix(confidentInterval) ;
      Console.WriteLine() ;

      Console.WriteLine("Error info:") ;
      String str_info = mwStructure_info.ToString() ;
      Console.WriteLine(str_info) ;

      objMatlab.WaitForFiguresToDie() ;

      // Free memory
      mw_X.Dispose() ;
      mw_Y.Dispose() ;

      MWNumericArray.DisposeArray(mw_ArrayOut) ;

      mwChar_formula.Dispose() ;     
      mwCell_coeffnames.Dispose() ;    
      mwDouble_coeffvalues.Dispose() ;
      mwDouble_confint.Dispose() ;  

      mwStructure_info.Dispose() ;   
    }

    /* **************************** */
    public void CurveFittingPlotsAdvance()
    {
      double[]X = {0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600} ;

      double[]Y = {0.4000, 0.2426, 0.1472, 0.0893, 0.0541, 0.0328, 0.0199, 
            0.0121, 0.0073, 0.0044, 0.0027, 0.0016,  0.0010 } ;

      MWNumericArray mw_X = new MWNumericArray(X) ;
      MWNumericArray mw_Y = new MWNumericArray(Y) ;

      MWArray [] mw_ArrayOut             = null ;

      MWCharArray mwChar_formula          = null ;
      MWCellArray mwCell_coeffnames       = null ;
      MWNumericArray mwDouble_coeffvalues = null ;
      MWNumericArray mwDouble_confint     = null ;

      MWStructArray mwStructure_info      = null ;

      String curveFitLib = "poly3" ;
      
      CurveFittings objMatlab = new CurveFittings() ;
      mw_X = (MWNumericArray)objMatlab.mytranspose(mw_X) ;
      mw_Y = (MWNumericArray)objMatlab.mytranspose(mw_Y) ;

      String graphicTitle = @"Curve Fitting Graphics" ;
      String xlabel = "Force" ;
      String ylabel = "Voltage" ;

      String curveFittingColor = "r" ;
      String curveDataColor  = "g";

      String curveFittingLegend = "My curve fitting" ;
      String dataLegend     = "My data" ;

      mw_ArrayOut = objMatlab.CurveFittingWithPlotsAdvance(5,mw_X, mw_Y, curveFitLib,
              graphicTitle,xlabel,ylabel, 
              curveFittingColor,curveDataColor,curveFittingLegend,dataLegend) ;
      
      mwChar_formula       = (MWCharArray) mw_ArrayOut[0] ;
      mwCell_coeffnames    = (MWCellArray) mw_ArrayOut[1] ;
      mwDouble_coeffvalues = (MWNumericArray) mw_ArrayOut[2] ;
      mwDouble_confint     = (MWNumericArray) mw_ArrayOut[3] ;
      mwStructure_info     = (MWStructArray)  mw_ArrayOut[4] ;

      // This output works as well
      /*
      Console.WriteLine(mwChar_formula    ) ;
      Console.WriteLine(mwCell_coeffnames   ) ;
      Console.WriteLine(mwDouble_coeffvalues) ;
      Console.WriteLine(mwDouble_confint  ) ;
      Console.WriteLine(mwStructure_info   ) ;
      */

      String str_formula = mwChar_formula.ToString() ;  
      Console.WriteLine("Curve fitting formula :") ;
      Console.WriteLine(str_formula) ;
      Console.WriteLine() ;

      String str_coeffnames = mwCell_coeffnames.ToString() ;
      Console.WriteLine("Coefficient names:") ;
      Console.WriteLine(str_coeffnames) ;
      Console.WriteLine() ;

      double[] db_coeffvalues = (double[])mwDouble_coeffvalues.ToVector(
                                                    MWArrayComponent.Real) ;
      Console.WriteLine("Coefficient values:") ;
      PrintVector(db_coeffvalues);
      Console.WriteLine() ;

      double[,] confidentInterval = (double[,])mwDouble_confint.ToArray(
                                                    MWArrayComponent.Real) ;
      Console.WriteLine("Confidence interval:");
      PrintMatrix(confidentInterval) ;
      Console.WriteLine() ;

      Console.WriteLine("Error info:") ;
      String str_info = mwStructure_info.ToString() ;
      Console.WriteLine(str_info) ;
      objMatlab.WaitForFiguresToDie() ;

      // Free memory
      mw_X.Dispose() ;
      mw_Y.Dispose() ;

      MWNumericArray.DisposeArray(mw_ArrayOut) ;

      mwChar_formula.Dispose() ;     
      mwCell_coeffnames.Dispose() ;    
      mwDouble_coeffvalues.Dispose() ;
      mwDouble_confint.Dispose() ;  

      mwStructure_info.Dispose() ;   
    }

    /* ******************************** */
    public static void PrintVector(double[] vector)
    {
      int i, row;
      row = vector.GetUpperBound(0) - vector.GetLowerBound(0) + 1;

      for (i = 0; i < row; i++)
      {
        Console.Write("{0} \t", vector.GetValue(i).ToString());
        Console.WriteLine();
      }
    }

    /* ******************************** */
    public static void PrintMatrix(double[,] matrix)  
    {  
      int i, j, row, col ;
      row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
      col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++) 
        {          
          Console.Write("{0} \t", matrix.GetValue(i,j).ToString() ) ;
        }
        Console.WriteLine() ;
      }
    }
    /* ******************************** */
    /* ******************************** */
    /* ******************************** */
  }
}
