﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using FFTNameSpace;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Program objProgram = new Program() ; 
      Console.WriteLine("Fast Fourier Transform.") ;

      Console.WriteLine("Fast Fourier Transform in 1D.") ;
      objProgram.FastFourierTrans1D() ;

      Console.WriteLine("Fast Fourier Transform in inverse of 1D.") ;
      objProgram.InverseFastFourierTrans1D() ;

      Console.WriteLine("Fast Fourier Transform in 2D.") ;
      objProgram.FastFourierTrans2D() ;

      Console.WriteLine("Fast Fourier Transform in inverse of 2D.") ;
      objProgram.InverseFastFourierTrans2D() ;
    }

    /* **************************** */
    public void FastFourierTrans1D()  
    {
      double[] db_X  = { 6, 3, 7, -9, 0, 3, -2, 1 } ; 
      int vectorSize = 8 ;

      /* declare mxArray variables */
      MWNumericArray mw_X = new MWNumericArray(db_X) ;
      MWNumericArray mw_Y = null ;

      /* call an implemental function */
      FFT objMatlab = new FFT() ;

      mw_Y = (MWNumericArray)objMatlab.myfft(mw_X) ;
      Console.WriteLine("Fast Fourier Transform of X : \n{0}", mw_Y) ;

      ///* convert back to Cs double  */
      double[] db_YReal = new double [vectorSize] ;
      double[] db_YImag = new double [vectorSize] ;

        db_YReal = (double[]) mw_Y.ToVector(MWArrayComponent.Real) ;

      try
      {
        db_YImag = (double[])mw_Y.ToVector(MWArrayComponent.Imaginary);
      }

      catch
      { 
         // nothing, db_YImag[] will be assigned value of zero 
      }

      /* print out */
      Console.WriteLine("Fast Fourier Transform of X : \n") ;
      int i ;
      for (i=0; i<vectorSize; i++)  
      {
        Console.Write( db_YReal[i].ToString() +  " + " ) ;
        Console.Write( db_YImag[i].ToString() +  "i"  + "\n" ) ;
      }
      
      Console.WriteLine("\n") ;

      mw_X.Dispose() ;
      mw_Y.Dispose() ;
    }

    /* **************************** */
    public void InverseFastFourierTrans1D()  
    {
      double[] db_YReal={ 9.00, 13.0711,   1.00, -1.0711, 13.00, -1.0711 , 1.00 , 13.0711};
      double[] db_YImag={ 0   , -1.9289, -14.00, 16.0711, 0    , -16.0711, 14.00, 1.9289 };
      int vectorSize = 8 ;

      /* declare mxArray variables */
      MWNumericArray mw_Y = new MWNumericArray(db_YReal, db_YImag) ;
      MWNumericArray mw_X = new MWNumericArray(MWArrayComplexity.Complex, 
                                                          MWNumericType.Double, 8) ;

      /* call an implemental function */
      FFT objMatlab = new FFT() ;
      mw_X = (MWNumericArray) objMatlab.myifft(mw_Y) ;
      Console.WriteLine("Inverse Fast Fourier Transform of Y : \n{0}", mw_X ) ;

      /* convert back to Cs double  */
      double[] db_XReal = (double[]) mw_X.ToVector(MWArrayComponent.Real)    ;
      double[] db_XImag = new double [vectorSize] ;

      try
      {
        db_XImag = (double[])mw_X.ToVector(MWArrayComponent.Imaginary);
      }
      catch
      {
        // nothing, db_XImag[] will be assigned value of zero
      }

      /* print out */
      Console.WriteLine("Inverse Fast Fourier Transform of Y : " ) ;
      int i ;
      for (i=0; i<vectorSize; i++)  
      {
        Console.Write( db_XReal[i].ToString() +  " + " ) ;
        Console.Write( db_XImag[i].ToString() +  "i" + "\n" ) ;
      }

      Console.WriteLine("\n") ;
      mw_X.Dispose() ;
      mw_Y.Dispose() ;
    }

    /* ***************************** */
    public void FastFourierTrans2D()  
    {
      double[,] X = { {4  ,  3.2,  6.8,  9.1  }  ,
                      {-4 ,  1.2,  4.3,  5.4  }  ,
                      {2.2, -6.7,  8  ,  12.2 }  } ;
      int row = 3 ;
      int col = 4 ;
      int i, j ;

      /* declare mxArray variables */
      MWNumericArray mw_X = new MWNumericArray (X) ;
      MWNumericArray mw_Y = null ;

      /* call an implemental function */
      FFT objMatlab = new FFT() ;
      mw_Y = (MWNumericArray) objMatlab.myfft2(mw_X) ;
      Console.WriteLine("2-D Fast Fourier Transform of X : \n{0}", mw_Y) ;

      /* convert back to Cs double  */
      double [,] db_YReal = new double [row, col] ;
      double [,] db_YImag = new double [row, col] ;

        db_YReal = (double[,]) mw_Y.ToArray(MWArrayComponent.Real)    ;

      try
      {
        db_YImag = (double[,])mw_Y.ToArray(MWArrayComponent.Imaginary);
      }
      catch
      {
        // nothing, db_YImag will be assigned value of zero
      }

      /* print out */
      Console.WriteLine("2-D Fast Fourier Transform of X : ") ;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++ ) 
        {
          Console.Write( db_YReal.GetValue(i,j).ToString() + " + " );
          Console.Write( db_YImag.GetValue(i,j).ToString() + "i" + "\t\t" ) ;
        }
        Console.WriteLine("\n") ;
      }
      Console.WriteLine("\n") ;

      /* free memory */
      mw_X.Dispose() ;
      mw_Y.Dispose() ;
    }

    /* **************************** */
    public void InverseFastFourierTrans2D()  
    {
      double[,] YReal = { { 45.7000,  -16.9000,  -3.1000,  -16.9000 }  ,
                          { 11.8000,   -8.4806,  -0.7000,   16.9806 }  ,
                          { 11.8000,   16.9806,  -0.7000,  -8.4806  }  };

      double[,] YImag =  {{ 0     ,  29.0000,   0     , -29.0000 }  ,
                          { 7.6210,  -3.4849,   9.5263,   7.8151 }  ,
                          {-7.6210,  -7.8151,  -9.5263,   3.4849 }  };
      int row = 3 ;
      int col = 4 ;
      int i, j ;

      /* declare mxArray variables */
      MWNumericArray mw_X = null ;
      MWNumericArray mw_Y = new MWNumericArray(YReal, YImag) ;

      /* call an implemental function */
      FFT objMatlab = new FFT() ;
      mw_X = (MWNumericArray) objMatlab.myifft2(mw_Y) ;
      Console.WriteLine("Inverse 2-D Fast Fourier Transform of Y : \n{0}", mw_X ) ;

      /* convert back to Cs double  */
      double[,] db_XReal = new double [row, col] ;
      double[,] db_XImag = new double [row, col] ;

        db_XReal = (double[,]) mw_X.ToArray(MWArrayComponent.Real)    ;

      try
      {
        db_XImag = (double[,])mw_X.ToArray(MWArrayComponent.Imaginary);
      }
      catch
      {
        // nothing, db_XImag will be assigned value of zero
      }

      /* print out */
      Console.WriteLine("Inverse 2-D Fast Fourier Transform of Y : " ) ;    

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++ ) 
        {
          Console.Write( db_XReal.GetValue(i,j).ToString() +  " + " ) ;
          Console.Write( db_XImag.GetValue(i,j).ToString() +  "i" + "\t\t" );
        }
        Console.WriteLine("\n") ;
      }
      Console.WriteLine("\n") ;

      /* free memory */
      mw_X.Dispose() ;
      mw_Y.Dispose() ;
    }

/* **************************** */
/* **************************** */
/* **************************** */
  }
}
