﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using FindingRootsNameSpace ;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Console.WriteLine("Roots of functions.") ;

      Program objProgram = new Program() ; 
      objProgram.FindingRootsPolynormial() ;

      Console.WriteLine("\n") ;
      objProgram.FindingZeroFunction() ;
    }

    /* **************************** */
    public void FindingRootsPolynormial()  
    {
      /* 
      Find the solutions of polynomial function:
          f(x) = -x^3 + 7.2x^2 -21x -5  
      */
      int order = 3 ;
      double[] db_coefs = { -1, 7.2, -21, -5 } ;

      /* declare mxArray variables */
      MWNumericArray mw_coefs = new MWNumericArray(db_coefs) ;
      MWNumericArray mw_x     = null ;

      /* call an implemental function */
      FindingRoots objMatlab = new FindingRoots() ;
      mw_x = (MWNumericArray) objMatlab.myroots(mw_coefs) ;
                  
      Console.WriteLine("Solutions of the polynomial function: \n {0}", mw_x) ;

      /* convert back to C/C++ double  */
      double []db_xReal = new double [order] ;
      double []db_xImag = new double [order] ;

      db_xReal = (double[]) mw_x.ToVector(MWArrayComponent.Real)      ;

      try
      {
          db_xImag = (double[])mw_x.ToVector(MWArrayComponent.Imaginary);
      }
      catch
      {
          // nothing, db_xImag[] will be assigned value of zero
      }

      /* print out */
      Console.WriteLine("Solutions of the polynomial function : ")    ;
      int i ;
      for (i=0; i<order; i++)  
      {
          Console.Write( db_xReal[i].ToString() +  " + " ) ;
          Console.Write( db_xImag[i].ToString() + "i \n" ) ;
      }

      /* free memory */
      mw_coefs.Dispose()  ;
      mw_x.Dispose()      ;
    }
         
    /* **************************** */
    public void FindingZeroFunction ( ) 
    {
      /* Find the solution of the function f(x) = sin(2*x) + cos(x) + 1  
          fzero(..) returns ONLY ONE SOLUTION near a initial guess value    
          If your problem is complicated, please look at functions 
          in Optimization Tool Box
      */
      double db_initialGuess = 0.9 ;
      String strfunc = "sin(2*x) + cos(x) + 1" ;

      /* declare mxArray variables */
      MWCharArray mw_strfunc  = new MWCharArray(strfunc)      ;
      MWArray [] mw_ArrayOut = null ;
      MWNumericArray mw_x     = null ;

      /* call an implemental function */
      FindingRoots objMatlab = new FindingRoots() ;

      mw_ArrayOut = objMatlab.myfzero(1, mw_strfunc, db_initialGuess) ;
      Console.WriteLine("Solutions of the given function :\n {0}", mw_ArrayOut ) ;

      mw_x = (MWNumericArray) mw_ArrayOut[0] ;
      Console.WriteLine("Solutions of the given function :\n {0}", mw_x ) ;            
     
      /* convert back to Cs double  */
      double db_xReal = (double) mw_x.ToScalarDouble() ;

      Console.WriteLine("Solutions of the given function : " ) ;
      Console.WriteLine( db_xReal.ToString() ) ;

      /* free memory */
      mw_strfunc.Dispose()    ;
      mw_x.Dispose()          ;

      MWNumericArray.DisposeArray(mw_ArrayOut) ;
    }

    /* ******************************** */
    /* ******************************** */
    /* ******************************** */
  }
}
