﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using IntegrationNameSpace;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Console.WriteLine("Single integration:");

      Program objProgram = new Program();
      objProgram.SingleIntegration();

      Console.WriteLine("Double-integration:");
      objProgram.DoubleIntegration();
    }

    /* ******************************************* */
    public void SingleIntegration()
    {
      String strfunc = "sin(x) + x.^2";

      double db_beginInterval = 0;
      double db_endInterval = 3 * Math.PI; // using the value pi in C#

      /* declare mxArray variables */
      MWCharArray mw_strfunc = new MWCharArray(strfunc);
      MWArray[] mw_ArrayOut = null;
      MWNumericArray mw_y = null;

      /* call an implemental function */
      Integration objMatlab = new Integration();


      /* Using Matlab function */
      mw_ArrayOut = objMatlab.myquad(1, mw_strfunc, db_beginInterval, db_endInterval);
      mw_y = (MWNumericArray)mw_ArrayOut[0];
      Console.WriteLine(" I = {0}", mw_ArrayOut);

      /* or */
      Console.WriteLine(" I = {0}", mw_y);

      /* or */
      double db_y = (double)mw_y;
      Console.WriteLine(" I = {0}", db_y);

      /* free memory */
      mw_strfunc.Dispose();
      mw_y.Dispose();

      MWNumericArray.DisposeArray(mw_ArrayOut);
    }

    /* ******************************************* */
    public void DoubleIntegration()
    {
      String strfunc = "sin(x) + x.^2 + y.^3";

      double db_x1 = 0;
      double db_x2 = 3 * Math.PI; // using the value pi in C#

      double db_y1 = 0;
      double db_y2 = Math.PI;

      /* declare mxArray variables */
      MWCharArray mw_strfunc = new MWCharArray(strfunc);
      MWArray[] mw_ArrayOut = null;
      MWNumericArray mw_II = null;

      /* call an implemental function */
      Integration objMatlab = new Integration();

      /* Using Matlab function */
      mw_ArrayOut = objMatlab.mydblquad(1, mw_strfunc, db_x1, db_x2, db_y1, db_y2);
      mw_II = (MWNumericArray)mw_ArrayOut[0];

      Console.WriteLine(" II = {0}", mw_ArrayOut);

      /* or */
      Console.WriteLine(" II = {0}", mw_II);

      /* or */
      double db_II = (double)mw_II;
      Console.WriteLine(" II = {0}", db_II);

      /* free memory */
      mw_strfunc.Dispose();
      mw_II.Dispose();

      MWNumericArray.DisposeArray(mw_ArrayOut);
    }

/* ******************************************* */
/* ******************************************* */
/* ******************************************* */
  }
}