﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using LinearSystemNameSpace;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Console.WriteLine(" Linear System Equations") ;

      Program objProgram = new Program() ; 
      objProgram.LinearSystemEquations() ;

      Console.WriteLine("\n LU_decompression") ;
      objProgram.LU_decompression() ;

      Console.WriteLine("\n Spare System") ;
      objProgram.SparseSystem() ;

      Console.WriteLine() ;
      objProgram.TridiagonalSystem() ;

      Console.WriteLine() ;
      objProgram.BandMatrixSystem() ;
    }

    /* **************************** */
    public void LinearSystemEquations()  
    {
      /* Solve general linear system equations Ax = b */
      double[,] A =  {    {1.1,  5.6, 3.3}  ,
                          {4.4, 12.3, 6.6}  ,
                          {7.7,  8.8, 9.9}  };

      double[] vectorb =  { 12.5, 32.2 , 45.6 } ;

      /* declare variables */
      MWNumericArray mw_A         = new MWNumericArray(A) ;
      MWNumericArray mw_vectorb   = new MWNumericArray(vectorb) ;
      MWNumericArray mw_x         = null ;

      /* call an implemental function */
      LinearSystem objLinear = new LinearSystem() ;
      mw_vectorb  = (MWNumericArray) objLinear.mytranspose(mw_vectorb) ;
      mw_x        = (MWNumericArray) objLinear.mymldivide(mw_A, mw_vectorb) ;

      /* convert back to Cs double  */
      double[] x = (double[])mw_x.ToVector(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("result=\n{0}", mw_x);

      /* or */
      PrintVector(x) ;

      /* free memory */
      mw_A.Dispose() ;
      mw_vectorb.Dispose() ;
      mw_x.Dispose() ;
    }

    /* **************************** */
    public void LU_decompression()  
    {
      /* find lower and upper matrixes */
      double[,] A =  {    {1.1,  5.6, 3.3}  ,
                          {4.4, 12.3, 6.6}  ,
                          {7.7,  8.8, 9.9}  };

      /* declare variables */
      MWNumericArray mw_A = new MWNumericArray(A) ;

      MWArray [] mw_ArrayOut = null ;
      MWNumericArray mw_L = null ;
      MWNumericArray mw_U = null ;
      MWNumericArray mw_P = null ;

      /* call an implemental function */
      // value of 3 is three of ouput in mylu.m

      LinearSystem objLinear = new LinearSystem() ;
      mw_ArrayOut = objLinear.mylu(3, mw_A) ;

      mw_L = (MWNumericArray) mw_ArrayOut[0] ;
      mw_U = (MWNumericArray) mw_ArrayOut[1] ;
      mw_P = (MWNumericArray) mw_ArrayOut[2] ;

      /* convert back to Cs double  */
      double [,] L = (double[,]) mw_L.ToArray(MWArrayComponent.Real) ;
      double [,] U = (double[,]) mw_U.ToArray(MWArrayComponent.Real) ;
      double [,] P = (double[,]) mw_P.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("The lower matrix") ;
      Console.WriteLine("L = \n{0}", mw_L)  ;

      Console.WriteLine("The upper matrix")   ;
      Console.WriteLine("U = \n{0}", mw_U)    ;

      Console.WriteLine("P = \n{0}", mw_P)    ;

      /* or */
      PrintMatrix(L) ;
      PrintMatrix(U) ;
      PrintMatrix(P) ;

      /* free memory */
      mw_A.Dispose() ;
      MWNumericArray.DisposeArray(mw_ArrayOut) ;
      mw_L.Dispose() ;
      mw_U.Dispose() ;
      mw_P.Dispose() ;
    }

    /* **************************** */
    public void SparseSystem()  
    {
      /*
      A = 0        0        0        0        1.1
          0        2.2      0        0        0    
          3.3      0        0        0        0
          0        0        0        6.6      0
          0        0        5.5      0        0

      b =  11.1    0        22.2     0        33.3
      */

      /* Solve general linear system equations Ax = b */
      double[,] A =    {  {0  ,    0  ,    0  ,   0  ,    1.1 } ,
                          {0  ,    2.2,    0  ,   0  ,    0   } ,
                          {3.3,    0  ,    0  ,   0  ,    0   } ,
                          {0  ,    0  ,    0  ,   6.6,    0   } ,
                          {0  ,    0  ,    5.5,   0  ,    0   } } ;

      double[] vectorb =  {11.1,   0  ,    22.2,  0  ,    33.3} ;

      /* declare variables */
      MWNumericArray mw_A         = new MWNumericArray(A) ;
      MWNumericArray mw_vectorb   = new MWNumericArray(vectorb) ;
      MWNumericArray mw_x         = null ;

      /* call an implemental function */
      LinearSystem objLinear = new LinearSystem() ;
      mw_vectorb = (MWNumericArray) objLinear.mytranspose(mw_vectorb) ;

      mw_A       = (MWNumericArray) objLinear.mysparse(mw_A) ;
      mw_vectorb = (MWNumericArray) objLinear.mysparse(mw_vectorb) ;

      mw_x       = (MWNumericArray) objLinear.mymldivide(mw_A, mw_vectorb) ;
      mw_x       = (MWNumericArray) objLinear.myfull(mw_x) ;

      /* convert back to Cs double  */
      double [] x = (double[]) mw_x.ToVector(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("Solution x :\n{0}", mw_x);

      /* or */
      PrintVector(x) ;

      /* free memory */
      mw_A.Dispose() ;
      mw_vectorb.Dispose() ;
      mw_x.Dispose() ;
    }
   
    /* **************************** */
    public void TridiagonalSystem()  
    {
      double [,]B = new double [6,6] ;

      /* columns 1 */ 
      B[0,0] = 1.1 ;
      B[1,0] = 1.2 ;
      B[2,0] = 1.3 ;
      B[3,0] = 1.4 ;
      B[4,0] = 1.5 ;
      B[5,0] = 1.6 ;

      /* columns 2 */ 
      B[0,1] = 4.1 ;
      B[1,1] = 4.2 ;
      B[2,1] = 4.3 ;
      B[3,1] = 4.4 ;
      B[4,1] = 4.5 ;
      B[5,1] = 4.6 ;

      /* columns 3 */
      B[0,2] = 2.1 ;
      B[1,2] = 2.2 ;
      B[2,2] = 2.3 ;
      B[3,2] = 2.4 ;
      B[4,2] = 2.5 ;
      B[5,2] = 2.6 ;

      double[] db_vectord = new double [6] ;
      db_vectord[0] = 1.2  ;
      db_vectord[1] = 4.5  ;
      db_vectord[2] = 5.6  ;
      db_vectord[3] = 12.4 ;
      db_vectord[4] = 7.8  ;
      db_vectord[5] = 6.8  ;

      // The size of the matrix A
      int row = 6 ;
      int col = 6 ;
      int band = 3;  /* tridiagnal, band width */

      // Establish the row size=m, and column size=n of the bufferA
      int m = row ;
      int n = col + (band-1) ;

      double[] d = { 0, 1, 2 }; /* for band=3, start from 0 */

      int rowB = row  ;
      int colB = band ;

      /* declare mxArray variables */
      MWNumericArray  mw_B        = new MWNumericArray(B) ;
      MWNumericArray  mw_bufferA  = null ;
      MWNumericArray  mw_A        = null ;

      MWNumericArray  mw_d        = new MWNumericArray(d) ;

      MWNumericArray  mw_vectord  = new MWNumericArray(db_vectord) ;
      MWNumericArray  mw_x        = null ;

      /* call an implemental function */
      LinearSystem objLinear = new LinearSystem() ;

      /* create a sparse matrix mw_bufferA from column-matrix B  */
      mw_bufferA = (MWNumericArray) objLinear.myspdiags(mw_B, mw_d, m, n);
      mw_bufferA = (MWNumericArray) objLinear.myfull(mw_bufferA) ;

      /* plot to see */
      Console.WriteLine("The buffer matrix A:") ;
      Console.WriteLine(mw_bufferA);

      /* extract the need-matrix from the buffter matrix, 
        (start at 1 following MATLAB in m.file )
         from row 1 to row 6(=m) and from column 2(=start+1) to column 7(=n-1) */
      mw_A = (MWNumericArray) objLinear.myextractmatrix(mw_bufferA, 1, row, 2, 7);

      /* plot to see */
      Console.WriteLine("The need-matrix A:") ;
      Console.WriteLine(mw_A);

      /* solve the tridiagnal system equations */
      mw_A       = (MWNumericArray) objLinear.mysparse(mw_A)               ;

      mw_vectord = (MWNumericArray) objLinear.mytranspose(mw_vectord)      ;
      mw_vectord = (MWNumericArray) objLinear.mysparse(mw_vectord)         ;

      mw_x       = (MWNumericArray) objLinear.mymldivide(mw_A, mw_vectord) ;
      mw_x       = (MWNumericArray) objLinear.myfull(mw_x)                 ;

      /* convert back to Cs double  */
      double [] x = (double[]) mw_x.ToVector(MWArrayComponent.Real) ;

      /*print out */
      Console.WriteLine("Tridiagnal system solution:") ;
      Console.WriteLine(mw_x);

      /* or */
      PrintVector(x) ;

      /* free memory */
      mw_B.Dispose() ;        
      mw_bufferA.Dispose() ;    
      mw_A.Dispose() ;        

      mw_d.Dispose() ;        

      mw_vectord.Dispose() ;    
      mw_x.Dispose() ;        
    }

    /* **************************** */
    public void BandMatrixSystem()  
    {
      double[,] B = new double[6,4] ;

      /* columns 1 */
      B[0,0] = 1.1 ;
      B[1,0] = 1.2 ;
      B[2,0] = 1.3 ;
      B[3,0] = 1.4 ;
      B[4,0] = 1.5 ;
      B[5,0] = 1.6 ;

      /* columns 2 */
      B[0,1] = 4.1 ;
      B[1,1] = 4.2 ;
      B[2,1] = 4.3 ;
      B[3,1] = 4.4 ;
      B[4,1] = 4.5 ;
      B[5,1] = 4.6 ;

      /* columns 3 */
      B[0,2] = 2.1 ;
      B[1,2] = 2.2 ;
      B[2,2] = 2.3 ;
      B[3,2] = 2.4 ;
      B[4,2] = 2.5 ;
      B[5,2] = 2.6 ;

      /* columns 4 */
      B[0,3] = 7.1 ;
      B[1,3] = 7.2 ;
      B[2,3] = 7.3 ;
      B[3,3] = 7.4 ;
      B[4,3] = 7.5 ;
      B[5,3] = 7.6 ;

      double [] db_vectord = new double [6] ;
      db_vectord[0] = 1.2  ;
      db_vectord[1] = 4.5  ;
      db_vectord[2] = 5.6  ;
      db_vectord[3] = 12.4 ;
      db_vectord[4] = 7.8  ;
      db_vectord[5] = 6.8  ;

      // The size of the matrix A
      int row = 6 ;
      int col = 6 ;

      int band = 4 ;  /* band width */

      // Establish the row size=m, and column size=n
      // of the bufferA

      int m = row ;
      int n = col + (band-1) ;

      double[] d = { 0, 1, 2, 3 }; /* for band = 4; start from 0 */

      int rowB = row  ;
      int colB = band ;

      /* declare mxArray variables */
      MWNumericArray  mw_B        = new MWNumericArray(B) ;
      MWNumericArray  mw_bufferA    = null ;
      MWNumericArray  mw_A        = null ;

      MWNumericArray  mw_d        = new MWNumericArray(d) ;

      MWNumericArray  mw_vectord    = new MWNumericArray(db_vectord) ;
      MWNumericArray  mw_x        = null ;

      /* call an implemental function */
      /* create a sparse matrix, size mxn, from column-matrix B  */
                  
      LinearSystem objLinear = new LinearSystem() ;

      mw_bufferA = (MWNumericArray) objLinear.myspdiags(mw_B, mw_d, m, n);
      mw_bufferA = (MWNumericArray) objLinear.myfull(mw_bufferA) ;

      /* plot to see */
      Console.WriteLine("The buffer band-matrix A:") ;
      Console.WriteLine(mw_bufferA);

      /* extract the need-matrix A from the buffter matrix,
          from row 1 to row 6 and from column 2 to column 7 */

      /* extract the need-matrix from the buffter matrix, 
        (start at 1 following MATLAB in m.file )
         from row 1 to row 6(=m) and from column 2(=start+1) to column 7(=n-1) */
      mw_A = (MWNumericArray) objLinear.myextractmatrix(mw_bufferA, 1, row, 2, 7);

      /* plot to see */
      Console.WriteLine("The band-need-matrix A:" ) ;
      Console.WriteLine(mw_A);

      /* solve the system equations */
      mw_A       = (MWNumericArray) objLinear.mysparse(mw_A)                  ;

      mw_vectord = (MWNumericArray) objLinear.mytranspose(mw_vectord)         ;
      mw_vectord = (MWNumericArray) objLinear.mysparse(mw_vectord)            ;

      mw_x       = (MWNumericArray) objLinear.mymldivide(mw_A, mw_vectord)    ;
      mw_x       = (MWNumericArray) objLinear.myfull(mw_x)                    ;

      /* convert back to Cs double  */
      double[] solution_x = (double[]) mw_x.ToVector(MWArrayComponent.Real) ;

      /*print out */
      /* convert back to Cs double */
      Console.WriteLine("Band matrix system solution:") ;
      Console.WriteLine(mw_x);

      /* or */
      PrintVector(solution_x) ;

      /* free memory */
      mw_B.Dispose() ;        
      mw_bufferA.Dispose() ;    
      mw_A.Dispose() ;        

      mw_d.Dispose() ;        

      mw_vectord.Dispose() ;    
      mw_x.Dispose() ;        
    }

    /* ******************************** */
    public static void PrintVector(double[] vector)
    {
      int i, row;
      row = vector.GetUpperBound(0) - vector.GetLowerBound(0) + 1;

      for (i = 0; i < row; i++)
      {
        Console.Write("{0} \t", vector.GetValue(i).ToString());
        Console.WriteLine();
      }
    }

    /* ******************************** */
    public static void PrintMatrix(double[,] matrix)  
    {    
      int i, j, row, col ;
      row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
      col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++) 
        {                    
            Console.Write("{0} \t", matrix.GetValue(i,j).ToString() ) ;
        }
        Console.WriteLine() ;
    }

    }

    /* ******************************** */
    /* ******************************** */
    /* ******************************** */
  }
}