using System;

// For MATLAB Version 7.9 (2009b) and MATLAB Compiler 4.11
namespace UtilityMatlabCSharpAPIVer4p11 
{
  class MatlabCSharpAPI  
  {
    // Instantiate MATLAB Engine Interface through COM
    public static MLApp.MLAppClass matlab = new MLApp.MLAppClass();

    /* ***************************** */
    /* ********** I. Scalar ******** */
    /* ***************************** */
    public static void PutScalarReal(String ml_a, double a)
    {
      matlab.PutWorkspaceData(ml_a, "base", (object) a) ;
    }
 
    /* ******************************** */
    public static void PlusComplexNumbers() {

      // 1. Set number
      double aReal = 1.1    ; 
      double aImag = 101.1  ;

      double bReal = 2.2    ;
      double bImag = 202.2  ;

      // 2. Perform tasks in Matlab Workspace
      matlab.PutWorkspaceData("ml_aReal", "base", aReal) ;
      matlab.PutWorkspaceData("ml_aImag", "base", aImag) ;
      matlab.Execute(" ml_aComp = complex(ml_aReal, ml_aImag) ; ") ;

      matlab.PutWorkspaceData("ml_bReal", "base", bReal) ;
      matlab.PutWorkspaceData("ml_bImag", "base", bImag) ;
      matlab.Execute(" ml_bComp = complex(ml_bReal, ml_bImag) ; ") ;

      matlab.Execute(" ml_cComp = ml_aComp + ml_bComp ; " ) ;
      
      matlab.Execute(" ml_cReal = real(ml_cComp) ; ") ;
      matlab.Execute(" ml_cImag = imag(ml_cComp) ; ") ;

      // 3. Get data from Matlab Workspace
      object obj_cReal, obj_cImag ;
      matlab.GetWorkspaceData("ml_cReal", "base", out obj_cReal) ;
      matlab.GetWorkspaceData("ml_cImag", "base", out obj_cImag) ;

      // 4. Print out
      double cReal = 0 ;
      double cImag = 0 ;

      cReal = Convert.ToDouble(obj_cReal) ;
      cImag = Convert.ToDouble(obj_cImag) ;

      Console.WriteLine("Result of a complex number :") ;
      Console.WriteLine("\t Real Part: {0}", cReal.ToString() ) ; 
      Console.WriteLine("\t Imag Part: {0}", cImag.ToString() ) ;

    }      
    
    /* ***************************** */
    public static void GetScalarReal(String ml_a, ref double a)
    {
      object obj_a ;
      matlab.GetWorkspaceData(ml_a, "base", out obj_a) ;

      a = Convert.ToDouble(obj_a) ;
    }
    
    /* ******************************** */
    /* *************II. Vector ******** */
    /* ******************************** */
    public static void PutVectorReal(String ml_vector, Array vector)
    {
      Array pi = new double [vector.Length] ; 
      Array.Clear(pi, 0, pi.Length);

      matlab.PutFullMatrix(ml_vector, "base", vector, pi) ;
    }

    /* ******************************** */
    public static void PutVectorComplex(String ml_vectorComplex, 
                                            Array vectorReal, Array vectorImag)
    {
      matlab.PutFullMatrix(ml_vectorComplex, "base", vectorReal, vectorImag) ;
    }

    /* ******************************** */
    public static void GetVectorReal(String ml_vector, ref Array vector)
    {
      Array pi = new double [vector.Length];
      matlab.GetFullMatrix(ml_vector, "base", ref vector, ref pi) ;
    }

    /* ******************************** */
    public static void GetVectorComplex(String ml_vectorComplex, ref Array vectorReal, 
                                                                 ref Array vectorImag)
    {
      matlab.GetFullMatrix(ml_vectorComplex, "base", ref vectorReal, ref vectorImag) ;
    }
 
    /* ******************************** */
    /* ********** III. Matrix ********* */
    /* ******************************** */
    public static void PutMatrixReal(String ml_matrix, Array matrix)
    {
    int row, col;
    row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
    col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      Array pi = Array.CreateInstance( typeof(double), row, col ) ;
      Array.Clear(pi, 0, pi.Length) ;

      matlab.PutFullMatrix(ml_matrix, "base", matrix, pi) ;
    }

    /* ******************************** */
    public static void PutMatrixComplex(String ml_matrixComplex, Array matrixReal, 
                                                                 Array matrixImag)
    {
      matlab.PutFullMatrix(ml_matrixComplex, "base", matrixReal, matrixImag) ;
    }

     /* ******************************** */
    public static void GetMatrixReal(String ml_matrix, ref Array matrix)
    {
    int row, col;
    row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
    col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      Array pi = Array.CreateInstance( typeof(double), row, col ) ;
      Array.Clear(pi, 0, pi.Length) ;

      matlab.GetFullMatrix(ml_matrix, "base", ref matrix, ref pi) ;
    }
        
    /* ******************************** */
    public static void GetMatrixComplex(String ml_matrixComplex, ref Array matrixReal, 
                                                                 ref Array matrixImag)
    {
      matlab.GetFullMatrix(ml_matrixComplex, "base", ref  matrixReal, ref matrixImag) ;
    }

    /* ******************************** */
    /* ************* Print out ******* */
    /* ******************************** */
    public static void PrintVector(Array vector)  
    {
      int i, row ;
      row = vector.GetUpperBound(0) - vector.GetLowerBound(0) + 1;

      for (i=0; i<row; i++)  
      {        
        Console.Write("{0} \t", (String) vector.GetValue(i).ToString() ) ;
        Console.WriteLine() ;
      }
    }

    /* ******************************** */
    public static void PrintMatrix(Array matrix)  
    {  
    int i, j, row, col ;
    row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
    col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

    for (i=0; i<row; i++)  
    {
    for (j=0; j<col; j++) 
    {      
      Console.Write("{0} \t", (String)matrix.GetValue(i,j).ToString() ) ;
    }
    Console.WriteLine() ;
    }
    }

    /* ******************************** */
    public static void Execute(String commandName)
    {
      matlab.Execute(commandName) ;
    }

    /* ******************************** */
    public static void PutString(String ml_str, String str)
    {
      matlab.PutCharArray(ml_str, "base", str) ;
    }

/* ******************************** */
/* ******************************** */
/* ******************************** */
  }
}