﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using UtilityMatlabCSharpAPIVer4p11 ;

namespace Example
{
  class Program
  {
    public MatlabCSharpAPI CSharpObj = new MatlabCSharpAPI() ;
  
    /* ******************************** */
    static void Main(string[] args)
    {
      Program progObj = new Program() ;

      // I . Scalar
      // 1. Put scalars in MATLAB Workspace
      // 2. Get a scalar from MATLAB Workspace
      Console.WriteLine() ;
      progObj.SimplePlus() ;

      Console.WriteLine() ;
      progObj.PlusComplexNumbers() ;

      
      // II. Vector
      // 1. Put a vector in MATLAB API
      // 2. Get a vecotr from MATLAB API
      Console.WriteLine() ;
      progObj.PlusRealVectors() ;

      Console.WriteLine() ;
      progObj.PlusComplexVectors() ;

      // III. Matrix
      // 1. Put a matrix in MATLAB  Workspace
      // 2. Get a matrix from MATLAB Workspace
      Console.WriteLine() ;
      progObj.LinearSolve() ;   
     
      // IV. Plot 
      // 1. Put a string in MATLAB Workspace
      // 2. Plot graphic from MATLAB Workspace
      Console.WriteLine() ;
      progObj.API_Plot() ;
      
    }

    /* ******************************** */
    public void SimplePlus()
    {
      double db_a = 1.1 ;
      double db_b = 2.2 ;

      MatlabCSharpAPI.PutScalarReal("ml_a", db_a ) ;
      MatlabCSharpAPI.PutScalarReal("ml_b", db_b ) ;

      MatlabCSharpAPI.Execute(" ml_c = ml_a + ml_b ; ") ;

      double db_c = 0 ;
      MatlabCSharpAPI.GetScalarReal("ml_c", ref db_c) ;

      //// print out
      Console.WriteLine("The result from simple addition: ") ;
      Console.WriteLine( db_c.ToString() ) ;
    }

    /* **************************** */
    public void PlusComplexNumbers()
    {
      MatlabCSharpAPI.PlusComplexNumbers() ;
    }

    /* **************************** */
    public void LinearSolve()  
    {
      double[,] A = { {1.1, 5.6, 3.3},  {4.4, 12.3, 6.6} , { 7.7, 8.8, 9.9} } ;
      double[]  b = { 12.5, 32.2 , 45.6 } ;

      // Put values to MATLAB variables
      MatlabCSharpAPI.PutMatrixReal("ml_A", A) ;
      MatlabCSharpAPI.PutVectorReal("ml_b", b) ;

      //MATLAB workspace tasks : Solve Ax = b, and get a vector x and an upper matrix U. 
      //The problem Ax = b is solved here as the purpose 
      //of showing the use of the MATLAB workspace in a C# function. 
      MatlabCSharpAPI.Execute(" ml_vectorX = mldivide(ml_A, ml_b) ; ") ;
      MatlabCSharpAPI.Execute(" [ml_L, ml_U, ml_P] = lu( ml_A )   ; ") ; 

      //Get results from MATLAB workspace
      Array vectorX = new double [b.Length] ; 
      MatlabCSharpAPI.GetVectorReal("ml_vectorX", ref vectorX) ; 

      int row, col;
			row = A.GetUpperBound(0) - A.GetLowerBound(0) + 1;
			col = A.GetUpperBound(1) - A.GetLowerBound(1) + 1;

      Array matrixU = Array.CreateInstance(typeof(double), row, col) ;
      MatlabCSharpAPI.GetMatrixReal("ml_U", ref matrixU) ;

      // Print out
      Console.WriteLine("Linear system: result of the vector solution") ;
      MatlabCSharpAPI.PrintVector(vectorX) ;

      Console.WriteLine() ;
      Console.WriteLine("Linear system: result of the U matrix") ;
      MatlabCSharpAPI.PrintMatrix(matrixU) ;
    
      // Or Convert to a double array
      Console.WriteLine("Section linear solve: Convert to double array: ") ;
      double[] db_vectorX = new double [b.Length] ; 
      vectorX.CopyTo(db_vectorX, 0) ;
      MatlabCSharpAPI.PrintVector(db_vectorX) ;

      // Or convert to a double matrix
      Console.WriteLine("Section Linear solve: Convert to double matrix") ;
      double[,] db_matrixU = new double[row, col] ;
      Array.Copy(matrixU, db_matrixU, row*col) ;
      MatlabCSharpAPI.PrintMatrix(db_matrixU) ;
    }	

    /* **************************** */
    public void PlusRealVectors()
    {
      double [] vecAReal = {  0.1,  1.1,   11.1 } ; 
      double [] vecBReal = {  0.2,  2.2,   22.2 } ;

      MatlabCSharpAPI.PutVectorReal("ml_vecA", vecAReal) ;
      MatlabCSharpAPI.PutVectorReal("ml_vecB", vecBReal) ;

      MatlabCSharpAPI.Execute( " ml_vecC = ml_vecA + ml_vecB ; " ) ;
      
      Array vectorC_Real = new double [vecAReal.Length] ;
      MatlabCSharpAPI.GetVectorReal("ml_vecC", ref vectorC_Real); 

      // Print out
      Console.WriteLine("Result of a real vector") ;
      MatlabCSharpAPI.PrintVector(vectorC_Real) ;
           
      // Or convert to a double array 
      double[] db_VectorC_Real = new double [vecAReal.Length] ;
      vectorC_Real.CopyTo(db_VectorC_Real,0) ;

      Console.WriteLine("Section real vector: Convert vector to a double array:") ;
      MatlabCSharpAPI.PrintVector(db_VectorC_Real) ;
    }

    /* **************************** */
    public void PlusComplexVectors()
    {
      double [] vecAReal = {  0.1,  1.1,   11.1 } ; 
      double [] vecAImag = {100.1, 101.1, 111.1 } ;

      double [] vecBReal = {  0.2,  2.2,   22.2 } ;
      double [] vecBImag = {200.2, 202.2, 222.2 } ;

      MatlabCSharpAPI.PutVectorComplex("ml_vecAComp", vecAReal, vecAImag) ;
      MatlabCSharpAPI.PutVectorComplex("ml_vecBComp", vecBReal, vecBImag) ;

      MatlabCSharpAPI.Execute( " ml_vecCComp = ml_vecAComp + ml_vecBComp ; " ) ;
      
      Array vectorC_Real = new double [vecAReal.Length] ;
      Array vectorC_Imag = new double [vecAImag.Length] ;

      MatlabCSharpAPI.GetVectorComplex("ml_vecCComp", ref vectorC_Real, ref vectorC_Imag); 

      // Print out
      Console.WriteLine("Result of a complex vector:") ;
      Console.WriteLine("Real vector:") ;      
      MatlabCSharpAPI.PrintVector(vectorC_Real) ;

      Console.WriteLine("Imag vector:") ;
      MatlabCSharpAPI.PrintVector(vectorC_Imag) ;
           
      // Convert to a double array 
      double[] db_VectorC_Real = new double [vecAReal.Length] ;
      double[] db_vectorC_Imag = new double [vecAImag.Length] ;

      vectorC_Real.CopyTo(db_VectorC_Real,0) ;
      vectorC_Imag.CopyTo(db_vectorC_Imag,0) ;

      Console.WriteLine("Section complex vector: Convert vector to a double array:") ;
      MatlabCSharpAPI.PrintVector(db_VectorC_Real) ;
      MatlabCSharpAPI.PrintVector(db_vectorC_Imag) ;      
    }

    /* ******************************** */
    public void API_Plot()
    {
      double[]X = {0, 50, 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600} ;

      double[]Y = {0.4000, 0.2426, 0.1472, 0.0893, 0.0541, 0.0328, 0.0199, 
             0.0121, 0.0073, 0.0044, 0.0027, 0.0016,  0.0010 } ;

      String myColor = "blue";

      MatlabCSharpAPI.PutVectorReal("ml_X", X) ;
      MatlabCSharpAPI.PutVectorReal("ml_Y", Y) ;

      MatlabCSharpAPI.PutString("ml_plotColor", myColor) ;
      
      MatlabCSharpAPI.Execute( "plot(ml_X, ml_Y, ml_plotColor); ") ;
      MatlabCSharpAPI.Execute( "grid on; ") ;

      Console.WriteLine("Hit Enter key to close the figure and continue.") ;
      Console.ReadLine() ; // Keep figure staying
    }

/* ******************************** */
/* ******************************** */
/* ******************************** */
  }
}
