﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using MatrixComputationsNameSpace ;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
        Console.WriteLine("Matrix Computations") ;
        Program objProgram = new Program() ; 
        Console.WriteLine() ;

        Console.WriteLine("Matrix addition") ;
        objProgram.AddMatrix() ;
        Console.WriteLine() ;

        Console.WriteLine("Matrix subtraction") ;
        objProgram.SubtractMatrix() ;
        Console.WriteLine() ;

        Console.WriteLine("Matrix multiplication") ;
        objProgram.MultipleMatrix();
        Console.WriteLine() ;

        Console.WriteLine("Matrix determinant") ;
        objProgram.DeterminantMatrix() ;
        Console.WriteLine() ;

        Console.WriteLine("Inverse matrix") ;
        objProgram.InverseMatrix() ;
        Console.WriteLine() ;

        Console.WriteLine("Transpose matrix") ;
        objProgram.TransposeMatrix() ;
        Console.WriteLine() ;
    }

    /* ***************************************** */
    public void AddMatrix() 
    {
      double [,]A = {{ 1.1, 2.2, 3.3} , {4.4, 5.5, 6.6} ,  {7.7, 8.8, 9.9} } ;
      double [,]B = {{ 11 , 12 , 13 } , {14 , 15 , 16 } ,  {17 , 18 , 19 } } ;

      /* declare variables */
      MWNumericArray mw_A = new MWNumericArray(A) ;
      MWNumericArray mw_B = new MWNumericArray(B) ;
      MWNumericArray mw_C = null ;

      /* call an implemental function */
      MatrixComputations objMatlab = new MatrixComputations() ;
      mw_C = (MWNumericArray)objMatlab.myplus(mw_A, mw_B) ;

      /* convert back to Cs double  */
      double [,] C = (double[,]) mw_C.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("result=\n{0}", mw_C);

      /* or */
      PrintMatrix(C) ;

      /* free memory */
      mw_A.Dispose() ;
      mw_B.Dispose() ;
      mw_C.Dispose() ;
    }

    /* ***************************************** */
    public void SubtractMatrix() 
    {
      double [,]A = {{ 1.1, 2.2, 3.3} , {4.4, 5.5, 6.6} ,  {7.7, 8.8, 9.9} } ;
      double [,]B = {{ 11 , 12 , 13 } , {14 , 15 , 16 } ,  {17 , 18 , 19 } } ;

      /* declare variables */
      MWNumericArray mw_A = new MWNumericArray(A) ;
      MWNumericArray mw_B = new MWNumericArray(B) ;
      MWNumericArray mw_C = null ;

      /* call an implemental function */
      MatrixComputations objMatlab = new MatrixComputations() ;
      mw_C = (MWNumericArray)objMatlab.myminus(mw_A, mw_B) ;

      /* convert back to Cs double  */
      double [,] C = (double[,]) mw_C.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("result=\n{0}", mw_C);

      /* or */
      PrintMatrix(C) ;

      /* free memory */
      mw_A.Dispose() ;
      mw_B.Dispose() ;
      mw_C.Dispose() ;
    }

    /* ***************************************** */
    public void MultipleMatrix() 
    {
      double [,]A = { { 1.1, 2.2  , 3.3  , 4.4   }    ,
                      { 5.5, 6.6  , 7.7  , 8.8   }    ,
                      { 9.9, 10.10, 11.11, 12.12 }    } ;

      double [,]B = {{ 10, 11}, {12, 13}, {14, 15}, {16, 17} } ;

      /* declare variables */
      MWNumericArray mw_A = new MWNumericArray(A) ;
      MWNumericArray mw_B = new MWNumericArray(B) ;
      MWNumericArray mw_C = null ;

      /* call an implemental function */
      MatrixComputations objMatlab = new MatrixComputations() ;
      mw_C = (MWNumericArray)objMatlab.mymtimes(mw_A, mw_B) ;

      /* convert back to Cs double  */
      double [,] C = (double[,]) mw_C.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("result=\n{0}", mw_C);

      /* or */
      PrintMatrix(C) ;

      /* free memory */
      mw_A.Dispose() ;
      mw_B.Dispose() ;
      mw_C.Dispose() ;
    }

    /* ***************************************** */
    public void DeterminantMatrix() 
    {
      double [,]A = {{ 1.1, 2.2, 3.3},  {7.7, 4.4, 9.9} , {4.4, 5.5, 8.8}  } ;

      /* declare variables */
      MWNumericArray mw_A     = new MWNumericArray(A) ;
      MWNumericArray mw_detA  = null ;

      /* call an implemental function */
      MatrixComputations objMatlab = new MatrixComputations() ;
      mw_detA = (MWNumericArray)objMatlab.mydet(mw_A) ;

      /* convert back to Cs double  */
      double detA = (double) mw_detA ;

      /* print out */
      Console.WriteLine("result = \n{0}", mw_detA) ;

      /* or */
      Console.WriteLine("result = \n{0}", detA) ;

      /* free memory */
      mw_A.Dispose()      ;
      mw_detA.Dispose()   ;
    }
    
    /* ***************************************** */
    public void InverseMatrix()
    {
      double [,]A = {{ -1 , 1 , 2} , {3 , -1 , 1} , {-1 , 3 , 4}  } ;

      /* declare variables */
      MWNumericArray mw_A         = new MWNumericArray(A) ;
      MWNumericArray mw_inverseA  = null ;

      /* call an implemental function */
      MatrixComputations objMatlab = new MatrixComputations() ;
      mw_inverseA = (MWNumericArray)objMatlab.myinv(mw_A) ;

      /* convert back to Cs double  */
      double[,] inverseA = (double[,])mw_inverseA.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("result = \n{0}", mw_inverseA);

      /* or */
      PrintMatrix(inverseA) ;

      /* free memory */
      mw_A.Dispose()          ;
      mw_inverseA.Dispose()   ;
    }        

    /* ***************************************** */
    public void TransposeMatrix() 
    {
      double [,] A = {{ -1 , 1 , 2} , {3 , -1 , 1} , {-1 , 3 , 4}  } ;

     /* declare variables */
      MWNumericArray mw_A             = new MWNumericArray(A) ;
      MWNumericArray mw_transposeA    = null ;

      /* call an implemental function */
      MatrixComputations objMatlab = new MatrixComputations() ;
      mw_transposeA = (MWNumericArray)objMatlab.mytranspose(mw_A) ;

      /* convert back to Cs double  */
      double[,] transposeA = (double[,])mw_transposeA.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("result = \n{0}", mw_transposeA);

      /* or */
      PrintMatrix(transposeA) ;

      /* free memory */
      mw_A.Dispose()          ;
      mw_transposeA.Dispose()   ;
    }
    
    /* ***************************************** */
    public static void PrintMatrix(double[,] matrix)  
    {    
      int i, j, row, col ;
      row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
      col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++) 
        {                    
          Console.Write("{0} \t", matrix.GetValue(i,j).ToString() ) ;
        }
        Console.WriteLine() ;
      }

    }

    /* ***************************************** */
    /* ***************************************** */
    /* ***************************************** */
    }
}
