﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using ODENameSpace ;

namespace Example
{
    class Program
    {
        static void Main(string[] args)
        {
            Program objProgram = new Program() ; 
            Console.WriteLine("ODE problems.") ;

            Console.WriteLine("First order ODE.") ;
            objProgram.FirstOrder() ;

            Console.WriteLine("First order ODE. Get particular values") ;
            objProgram.FirstOrderGetParticularValues() ;

            Console.WriteLine("Second order ODE.") ;
            objProgram.SecondOrder()  ;
        }

        /* **************************** */
        public void FirstOrder()  
        {
            /* Calculating first order ODE */ 
            //String strfunc = "2+y" ;

            String strfunc = "cos(t)" ;

            double db_y0 = 2.2    ; /* initial condition at t0        */

            double []db_tspan = new double [2]    ;
            db_tspan[0] = 0.2    ; /* begin interval t0 = 0.2        */
            db_tspan[1] = 6.5    ; /* end interval, we choose this    */

            /* declare mxArray variables */
            MWCharArray mw_strfunc  = new MWCharArray(strfunc)      ;
            MWNumericArray mw_tspan = new MWNumericArray(db_tspan)  ;
            MWArray [] mw_ArrayOut  = null ;
            MWNumericArray mw_t     = null ;
            MWNumericArray mw_y     = null ;

            /* call an implemental function */
            ODE objMatlab = new ODE() ;

            /* mx_tspan is a column vector when using in the following function */
            /* call an implemental function */
            mw_ArrayOut = objMatlab.myode45firstorder(2, mw_strfunc, mw_tspan, db_y0) ;
            mw_t        = (MWNumericArray) mw_ArrayOut[0] ;
            mw_y        = (MWNumericArray) mw_ArrayOut[1] ;

            /* convert back to Cs double  */
            double[] db_t = (double[])mw_t.ToVector(MWArrayComponent.Real);
            double[] db_y = (double[])mw_y.ToVector(MWArrayComponent.Real);

            Console.WriteLine("The column of time");
            Console.WriteLine(mw_t) ;

            /* or */
            PrintVector(db_t) ;

            Console.WriteLine("The column of the function values y") ;
            Console.WriteLine(mw_y) ;
                        
            /* or */
            PrintVector(db_y) ;

            /* free memory */
            mw_strfunc.Dispose() ; 
            mw_tspan.Dispose() ; 
            mw_t.Dispose() ;     
            mw_y.Dispose() ;     

            MWNumericArray.DisposeArray(mw_ArrayOut) ;
        }

        /* **************************** */
        public void FirstOrderGetParticularValues()  
        {
            /* Calculating first order ODE */ 
            String strfunc = "6.4*t.^2 - 3.8*t*y" ;
            double db_y0 = 1.24  ; /* initial condition at t0            */

            double[] db_tspan = new double [4]  ;
            db_tspan[0] = 0.15  ; /* begin interval t0 = 0.15           */
            db_tspan[1] = 0.2   ; /* choose a particular time t = 0.2   */
            db_tspan[2] = 2.6   ; /* choose a particular time t = 2.6   */
            db_tspan[3] = 5.0   ; /* choose a particular time t = 5.0   */

            MWCharArray mw_strfunc  = new MWCharArray(strfunc)      ;
            MWNumericArray mw_tspan = new MWNumericArray(db_tspan)  ;

            MWArray [] mw_ArrayOut  = null ;
            MWNumericArray mw_t     = null ;
            MWNumericArray mw_y     = null ;

            /* mx_tspan is a column vector when using in the following function */
            /* convert Cs double to mxArray */

            /* call an implemental function */
            ODE objMatlab = new ODE() ;
            mw_ArrayOut = objMatlab.myode45firstorder(2, mw_strfunc, mw_tspan, db_y0) ;
            mw_t        = (MWNumericArray) mw_ArrayOut[0] ;
            mw_y        = (MWNumericArray) mw_ArrayOut[1] ;

            /* convert back to Cs double  */
            double[] db_t = (double[])mw_t.ToVector(MWArrayComponent.Real);
            double[] db_y = (double[])mw_y.ToVector(MWArrayComponent.Real);

            Console.WriteLine("The column of time") ;
            Console.WriteLine(mw_t) ;

            /* or */
            PrintVector(db_t) ;

            Console.WriteLine("The column of the function values y" ) ;
            Console.WriteLine(mw_y) ;

            /* or */
            PrintVector(db_y);

            /* free memory */
            mw_strfunc.Dispose() ; 
            mw_tspan.Dispose() ; 
            mw_t.Dispose() ;     
            mw_y.Dispose() ;     

            MWNumericArray.DisposeArray(mw_ArrayOut) ;
        }

        /* **************************** */
        public void SecondOrder()  
        {
            /* Calculating second order ODE */
            String strfunc = "cos(3*t) + 2*yprime + 6*y" ;

            double[] db_ybc = new double [2]    ; /* y boundary conditions            */
            db_ybc[0]  = 0.2    ; /* initial condition of y  at t0    */
            db_ybc[1]  = 1.1    ; /* initial condition of y' at t0    */

            double[] db_tspan = new double [2]    ;
            db_tspan[0] = 1.2    ; /* begin interval t0 = 1.2        */
            db_tspan[1] = 2.5    ; /* end interval, we choose this    */

            /* declare mxArray variables */
            MWCharArray mw_strfunc  = new MWCharArray(strfunc)      ;
            MWNumericArray mw_tspan = new MWNumericArray(db_tspan)  ;
            MWNumericArray mw_ybc   = new MWNumericArray(db_ybc)    ;

            MWArray [] mw_ArrayOut  = null ;
            MWNumericArray mw_t     = null ;
            MWNumericArray mw_y     = null ;

            /* call an implemental function */
            ODE objMatlab = new ODE() ;
            mw_ArrayOut = objMatlab.myode45secondorder(2, mw_strfunc, mw_tspan, mw_ybc) ;
            mw_t        = (MWNumericArray) mw_ArrayOut[0] ;
            mw_y        = (MWNumericArray) mw_ArrayOut[1] ;            

            /* convert back to Cs double  */
            double[] db_t  = (double[])  mw_t.ToVector(MWArrayComponent.Real);
            double[,] db_y = (double[,]) mw_y.ToArray(MWArrayComponent.Real) ;

            Console.WriteLine("The column of time") ;
            Console.WriteLine(mw_t) ;

            /* or */
            // PrintVector(db_t) ;

            Console.WriteLine("The column of the function values y :") ;
            /* first column of the matrix y */
            int i ;
            int aLength = db_t.Length ;
            for (i=0; i<aLength; i++) 
            {
                Console.Write("{0} ", db_y.GetValue(i,0).ToString() ) ;
                Console.WriteLine() ;
            }

            Console.WriteLine("The column of the first derivative y' :") ;
            /* second column of the matrix y */
            for (i=0; i<aLength; i++)
            {
                Console.Write("{0} ", db_y.GetValue(i,1).ToString() ) ;
                Console.WriteLine() ;
            }

            /* free memory */
            mw_strfunc.Dispose() ; 
            mw_tspan.Dispose() ; 
            mw_ybc.Dispose() ;
            mw_t.Dispose() ;     
            mw_y.Dispose() ;     

            MWNumericArray.DisposeArray(mw_ArrayOut) ;
        }

        /* ******************************** */
        public static void PrintVector(double[] vector)
        {
            int i, row;
            row = vector.GetUpperBound(0) - vector.GetLowerBound(0) + 1;

            for (i = 0; i < row; i++)
            {
                Console.Write("{0} \t", vector.GetValue(i).ToString());
                Console.WriteLine();
            }
        }

        /* **************************** */
        /* **************************** */
        /* **************************** */
    }
}