﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using PolyInterpolationNameSpace;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Console.WriteLine("Curve Fitting") ;
      Program objProgram = new Program() ; 

      Console.WriteLine() ;
      Console.WriteLine("1. Polynomial") ;
      objProgram.PolynomialFittingCurve() ;
  
      Console.WriteLine() ;
      Console.WriteLine("2. One-dimensional interpolation" ) ;
      objProgram.OneDimensionInterpolation() ;
      
      Console.WriteLine() ;
      Console.WriteLine("3. Two-dimensional interpolation") ;
      objProgram.TwoDimensionsInterpolation() ;

      Console.WriteLine() ;
      Console.WriteLine("4. Two-dimensional interpolation with fine grid") ;
      objProgram.TwoDimensionsInterpolationFineSolution() ;
    }

    /* ******************** */
    public void PolynomialFittingCurve()  
    {
      double[] db_X = { 1, 2, 3, 4, 5, 6 } ;
      double[] db_Y= { 6.8, 50.2, 140.8, 280.5, 321.4, 428.6 };

      double db_oneValue  = 2.2  ;

      /* declare mwArray variables */
      MWNumericArray mw_X         = new MWNumericArray(db_X) ;
      MWNumericArray mw_Y         = new MWNumericArray(db_Y) ;
      MWNumericArray mw_coefs     = null ;
      MWNumericArray mw_funcValue = null ;

      /* call an implemental function */
      /* mw_coefs is a matrix with row = 1 */
      PolyInterpolation obj = new PolyInterpolation() ;
      mw_coefs = (MWNumericArray) obj.mypolyfit(mw_X, mw_Y, 3) ;
        
      /* convert back to Cs double  */
      double[] db_coefs = (double[]) mw_coefs.ToVector(MWArrayComponent.Real) ;

      /* print out */
      Console.Write("The coefficient value: {0} \n", mw_coefs ) ;      

      /* print out */
      Console.WriteLine("The polynomial:" ) ;
      Console.Write( " {0}x^3 + ",  db_coefs[0].ToString() ) ;
      Console.Write( " {0}x^2 + ",  db_coefs[1].ToString() ) ;
      Console.Write( " {0}x + "  ,  db_coefs[2].ToString() ) ;
      Console.Write(                db_coefs[3].ToString() ) ;
      Console.WriteLine() ;

      /* calculate the function value at oneValue */
      mw_funcValue = (MWNumericArray) obj.mypolyval(mw_coefs, db_oneValue) ;

      double db_funcValue = (double)mw_funcValue ;
        
      Console.Write("The function value at 2.2 is: {0} \n", mw_funcValue ) ;      
      Console.Write("The function value at 2.2 is: {0} \n", 
                                                 db_funcValue.ToString() ) ;      
      /* free memory */
      mw_X.Dispose()       ; 
      mw_Y.Dispose()       ; 
      mw_coefs.Dispose()     ; 
      mw_funcValue.Dispose()   ; 
    }

    /* *************************** */
    public void OneDimensionInterpolation()  
    {
      double[] db_X = { 1, 2, 3, 4, 5, 6, 7, 8 } ;
      double[] db_Y = { 6.8, 24.6, 50.2, 74, 140.8, 280.5, 321.4, 428.6 };

      double db_oneValue  = 2.1  ;

      /* declare mwArray variables */
      MWNumericArray mw_X  = new MWNumericArray(db_X) ;
      MWNumericArray mw_Y  = new MWNumericArray(db_Y) ;
      MWNumericArray mw_funcValue = null ;

      /* call an implemental function */
      PolyInterpolation obj = new PolyInterpolation() ;
      mw_funcValue = (MWNumericArray) obj.myinterp1(mw_X, mw_Y, db_oneValue) ;

      /* print out */
      Console.WriteLine( "The function value at 2.1 is: {0}", mw_funcValue );

      /* convert back to Cs double  */
      double db_funcValue = (double) mw_funcValue ;
      Console.WriteLine( "The function value at 2.1 is: {0}",
                                              db_funcValue.ToString() );
      /* free memory */
      mw_X.Dispose()       ; 
      mw_Y.Dispose()       ; 
      mw_funcValue.Dispose()   ; 
    }

    /* **************************** */
    public void TwoDimensionsInterpolation()  
    {
      /* function values z at 47 points, (x_i, y_j) are : */
      /* matrix z values*/
    double[,] z = {
    {  0.0001,  0.0034, -0.0299, -0.2450, -0.1100, -0.0043,  0.0000  }  ,
    {  0.0007,  0.0468, -0.5921, -4.7596, -2.1024, -0.0616,  0.0004  }  ,
    { -0.0088, -0.1301,  1.8559, -0.7239, -0.2729,  0.4996,  0.0130  }  ,
    { -0.0365, -1.3327, -1.6523,  0.9810,  2.9369,  1.4122,  0.0331  }  ,
    { -0.0137, -0.4808,  0.2289,  3.6886,  2.4338,  0.5805,  0.0125  }  ,
    {  0.0000,  0.0797,  2.0967,  5.8591,  2.2099,  0.1328,  0.0013  }  ,
    {  0.0000,  0.0053,  0.1099,  0.2999,  0.1107,  0.0057,  0.0000  }  };

      double[] db_vectorstep = {-3, 1, 3} ;
      double db_a = 2.3 ;
      double db_b = 0.7 ;

      /* declare mwArray variables */
      String interp2method = "cubic";

      MWNumericArray  mw_interp2z  = null  ;
      MWNumericArray  mw_z  = new MWNumericArray(z)  ;
      MWNumericArray  mw_x  = null  ;
      MWNumericArray  mw_y  = null  ;

      /* same for two step-vectors */
      MWNumericArray  mw_vectorstepx  = new MWNumericArray(db_vectorstep) ;
      MWNumericArray  mw_vectorstepy  = new MWNumericArray(db_vectorstep) ;

      MWCharArray   mw_method = new MWCharArray(interp2method)  ;
      MWArray [] mw_ArrayGrid = null ;

      /* call implemental functions */
      PolyInterpolation obj = new PolyInterpolation() ;
      
      /* create values for the matrix x and matrix y */
      mw_ArrayGrid = obj.mymeshgrid(2, mw_vectorstepx, mw_vectorstepy) ;

      mw_x = (MWNumericArray) mw_ArrayGrid[0] ;
      mw_y = (MWNumericArray) mw_ArrayGrid[1] ;

      mw_interp2z = (MWNumericArray) obj.myinterp2(mw_x, mw_y, mw_z, db_a, db_b, mw_method);

      /* convert back to Cs double  */
      double[,] db_x = (double[,]) mw_x.ToArray(MWArrayComponent.Real) ;
      double[,] db_y = (double[,]) mw_y.ToArray(MWArrayComponent.Real) ;

      /* print out */
      Console.WriteLine("Matrix x {0}: \n", mw_x ) ;
      
      /* or */
      PrintMatrix(db_x) ;

      Console.WriteLine("Matrix y {0}: \n", mw_y ) ;
      
      /* or */
      PrintMatrix(db_y) ;

      Console.WriteLine("Interpolation in two dimensions with cubic method " ) ;    
      Console.WriteLine(" z = {0}", mw_interp2z ); 

      double db_interp2z = (double) mw_interp2z ;
      Console.WriteLine(" z = {0}", db_interp2z.ToString() ); 

      /* free memory */
      mw_interp2z  = null  ;
      mw_z.Dispose() ;
      mw_x.Dispose() ;
      mw_y.Dispose() ;
      mw_vectorstepx.Dispose() ;
      mw_vectorstepy.Dispose() ;
      mw_method.Dispose() ;

      MWNumericArray.DisposeArray(mw_ArrayGrid) ;
    }

    /* **************************** */
    public void TwoDimensionsInterpolationFineSolution()  
    {
      /* function values z, (x_i, y_j) are : */
      /* matrix z values*/

      double[,] z = {
  {  0.0001,  0.0034, -0.0299, -0.2450, -0.1100, -0.0043,  0.0000  }  ,
  {  0.0007,  0.0468, -0.5921, -4.7596, -2.1024, -0.0616,  0.0004  }  ,
  { -0.0088, -0.1301,  1.8559, -0.7239, -0.2729,  0.4996,  0.0130  }  ,
  { -0.0365, -1.3327, -1.6523,  0.9810,  2.9369,  1.4122,  0.0331  }  ,
  { -0.0137, -0.4808,  0.2289,  3.6886,  2.4338,  0.5805,  0.0125  }  ,
  {  0.0000,  0.0797,  2.0967,  5.8591,  2.2099,  0.1328,  0.0013  }  ,
  {  0.0000,  0.0053,  0.1099,  0.2999,  0.1107,  0.0057,  0.0000  }  };

      double[] db_vectorstep     = {-3,  1 , 3} ; // interval = 1
      double[] db_finevectorstep = {-3, 0.2, 3} ; // interval = 0.2

      double db_a = 2.3 ;
      double db_b = 0.7 ;
      String interp2method = "cubic";

      /* declare mwArray variables */
      MWNumericArray  mw_interp2z  = null  ;
      MWNumericArray  mw_z    = new MWNumericArray(z)  ;
      MWNumericArray  mw_finez  = new MWNumericArray(z)  ;

      MWNumericArray  mw_x    = null  ;
      MWNumericArray  mw_y    = null  ;

      MWNumericArray  mw_finex  = null  ;
      MWNumericArray  mw_finey  = null  ;

      /* same for two step-vectors */
      MWNumericArray  mw_vectorstepx  = new MWNumericArray(db_vectorstep) ;
      MWNumericArray  mw_vectorstepy  = new MWNumericArray(db_vectorstep) ;

      MWCharArray   mw_method     = new MWCharArray(interp2method)  ;

      MWNumericArray  mw_finevectorstepx = new MWNumericArray(db_finevectorstep)  ;
      MWNumericArray  mw_finevectorstepy = new MWNumericArray(db_finevectorstep)  ;

      MWArray [] mw_ArrayGridXY     = null ;

      MWArray [] mw_ArrayMatrixXY     = null ;
      MWArray [] mw_ArrayMatrixXY_fine  = null ;

      MWArray [] mw_ArrayGridXY_fine  = null ;
      MWArray [] mw_ArrayFineVector   = null ;     

      /* call implemental functions */
      /* get size for fine matrixes */
      PolyInterpolation obj = new PolyInterpolation() ;

      /* create values for the matrix x and matrix y */
      mw_ArrayMatrixXY = obj.mymeshgrid(2, mw_vectorstepx, mw_vectorstepy) ;
      mw_x = (MWNumericArray) mw_ArrayMatrixXY[0] ;
      mw_y = (MWNumericArray) mw_ArrayMatrixXY[1] ;

      //Console.WriteLine(" see normal x = {0} \n",  mw_x ) ; 

      /* create values for the fine matrix x and fine matrix y */
      mw_ArrayMatrixXY_fine = obj.mymeshgrid(2, mw_finevectorstepx, mw_finevectorstepy) ;
      mw_finex = (MWNumericArray) mw_ArrayMatrixXY_fine[0] ;
      mw_finey = (MWNumericArray) mw_ArrayMatrixXY_fine[1] ;

      //Console.WriteLine(" see fine x = {0}",  mw_finex ) ; 

      /* get a fine matrix mx_finez from mx_z */
      mw_finez = (MWNumericArray) obj.mygriddata(mw_x, mw_y, mw_z, mw_finex, mw_finey);

      //Console.WriteLine(" see fine z = {0} \n",  mw_finez ) ; 

      mw_interp2z = (MWNumericArray) obj.myinterp2(mw_finex, mw_finey, mw_finez, db_a, 
                                                                     db_b, mw_method);
      /* convert back to Cs double  */
      double db_interp2finez = (double) mw_interp2z ;
      
      Console.Write(@"Interpolation in two dimensions with cubic method ");
      Console.WriteLine(@"and a fine grid");
      Console.WriteLine(" z = {0}",  mw_interp2z ) ; 
      Console.WriteLine(" z = {0}",  db_interp2finez.ToString() ) ; 

      /* free memory */
      mw_interp2z.Dispose() ;
      mw_z.Dispose() ;
      mw_finez.Dispose() ;

      mw_x.Dispose() ;
      mw_y.Dispose() ;

      mw_vectorstepx.Dispose() ;
      mw_vectorstepy.Dispose() ;
      mw_method.Dispose() ;

      mw_finevectorstepx.Dispose() ;
      mw_finevectorstepy.Dispose() ;

      MWNumericArray.DisposeArray(mw_ArrayGridXY)     ;
      MWNumericArray.DisposeArray(mw_ArrayGridXY_fine)  ;
      MWNumericArray.DisposeArray(mw_ArrayFineVector)   ;  
    }

    /* ***************************************** */
    public static void PrintMatrix(double[,] matrix)  
    {
      int i, j, row, col ;
      row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
      col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++) 
        {      
          Console.Write("{0} \t", matrix.GetValue(i,j).ToString() ) ;
        }
        Console.WriteLine() ;
      }
    }

/* ***************************************** */
/* ***************************************** */
/* ***************************************** */
  }
}
