﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using MathWorks.MATLAB.NET.Arrays;
using MathWorks.MATLAB.NET.Utility;
using RandomNumberNameSpace ;

namespace Example
{
  class Program
  {
    static void Main(string[] args)
    {
      Console.WriteLine("Random Number") ;
      Program objProgram = new Program() ; 
      
      Console.WriteLine() ;
      Console.WriteLine("1. Vector of uniform random number from 0 to 1") ;
      objProgram.UniformRandom_vector() ;

      Console.WriteLine() ;
      Console.WriteLine("2. Vector of uniform random numbers from a to b ") ;
      objProgram.UniformRandom_vector2() ;

      Console.WriteLine() ;
      Console.WriteLine("3. Matrix of random numbers") ;
      objProgram.UniformRandom_matrix() ;

      Console.WriteLine() ;
      Console.WriteLine("4. Matrix of random numbers from a to b") ;
      objProgram.UniformRandom_matrix2() ;
      
      Console.WriteLine() ;
      Console.WriteLine("5. Vector of normal random number") ;
      objProgram.NormalRandom_vector() ;

      Console.WriteLine() ;
      Console.WriteLine("6. Vector of normal random number with mean and variance") ;
      objProgram.NormalRandom_vector2() ;

      Console.WriteLine() ;
      Console.WriteLine("7. Matrix of normal random number") ;
      objProgram.NormalRandom_matrix() ;
      
      Console.WriteLine() ;
      Console.WriteLine("8. Matrix of normal random number with mean and variance") ;
      objProgram.NormalRandom_matrix2() ;
    }

    /* **************************** */
    public void UniformRandom_vector()  
    {
      int row = 1 ;
      int col = 5 ;
      
      MWNumericArray uniformRandVector = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;
       
      RandomNumber objMatlab = new RandomNumber() ;      
      uniformRandVector = (MWNumericArray) objMatlab.myrand(mw_row, mw_col);

      Console.WriteLine("Uniform random numbers from 0 to 1:");
      Console.WriteLine(uniformRandVector);

      // Or convert back to double  
      double[] db_uniformRandVector = (double[])uniformRandVector.ToVector(
                                                          MWArrayComponent.Real) ;
      Console.WriteLine() ;
      PrintVector(db_uniformRandVector) ;

      uniformRandVector.Dispose() ;             
    }  

    /*  *********************************** */
    public void UniformRandom_vector2()  
    {
      double a = 2.0	;
      double b = 18.0	;

      int row = 1 ;
      int col = 6 ;
      int i ;
            
      // Get random numbers in [0, 1] 
      MWNumericArray uniformRandVector = null ;

      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      RandomNumber objMatlab = new RandomNumber() ;         
      uniformRandVector = (MWNumericArray) objMatlab.myrand(mw_row, mw_col);
        
      // Convert back to double 
      double[] db_uniformRandVector = (double[])uniformRandVector.ToVector(
                                                             MWArrayComponent.Real) ;
      
      Console.WriteLine() ;
      Console.WriteLine("Uniform random numbers from a=2 to b=18 : ") ;

      for (i=0; i<db_uniformRandVector.Length; i++)  {
        double aVal = a + (b-a)*db_uniformRandVector[i] ;
        Console.WriteLine( aVal.ToString() ) ;
      }

      // Free memories        
      uniformRandVector.Dispose() ;
    }

    /* ************************************* */
    public void UniformRandom_matrix()  
    {
      int row = 8 ;
      int col = 5 ;

      MWNumericArray mw_uniformRandMatrix = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      RandomNumber objMatlab = new RandomNumber() ;
      mw_uniformRandMatrix = (MWNumericArray)objMatlab.myrand(mw_row, mw_col);

      Console.WriteLine("The matrix of uniform random numbers from 0 to 1 :" ) ;
      Console.WriteLine(mw_uniformRandMatrix);
      
      // Or convert back to  double 
      double [,] db_uniformRandMatrix = (double[,])mw_uniformRandMatrix.ToArray(
                                                                MWArrayComponent.Real) ;
      Console.WriteLine() ;
      PrintMatrix(db_uniformRandMatrix) ;

      mw_uniformRandMatrix.Dispose() ;
    }

    /* ********************************* */
    public void UniformRandom_matrix2()  
    {
      int row = 8 ;
      int col = 5 ;
      int i, j ;

      double a = 4.0  ;
      double b = 17.0 ;

      MWNumericArray mw_uniformRandMatrix = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      // Get matrix of random number from 0 to 1
      RandomNumber objMatlab = new RandomNumber() ;
      mw_uniformRandMatrix = (MWNumericArray)objMatlab.myrand(mw_row, mw_col);

      // Convert to double
      double[,] db_uniformRandMatrix = (double[,])mw_uniformRandMatrix.ToArray(
                                                                MWArrayComponent.Real) ;

      // Print out 
      Console.WriteLine() ;
      Console.WriteLine("The matrix of uniform random numbers from a=4.0 to b=17.0:") ;

      for (i=0; i<row; i++)  {
        for (j=0; j<col; j++)  {
          Console.WriteLine( a + (b-a)*db_uniformRandMatrix[i,j] + "\t");
        }
        Console.WriteLine() ;
      }

      Console.WriteLine() ;

      // Free memories  
      mw_uniformRandMatrix.Dispose() ;
    }

    /* ********************************* */
    public void NormalRandom_vector()  
    {
      int row = 1 ;
      int col = 5 ;

      MWNumericArray mw_normalRandVector = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      RandomNumber objMatlab = new RandomNumber() ;      
      mw_normalRandVector = (MWNumericArray)objMatlab.myrandn(mw_row, mw_col) ;

      Console.WriteLine("Normal random numbers :") ;
      Console.WriteLine(mw_normalRandVector) ;

      // Or convert back to double  
      double [] db_normalRandVector = (double[])mw_normalRandVector.ToVector(
                                                                 MWArrayComponent.Real) ;
      Console.WriteLine() ;
      PrintVector(db_normalRandVector) ;

      // Free memories 
      mw_normalRandVector.Dispose() ;     
    }

    /* **************************** */
    public void NormalRandom_vector2()  
    {
      /* Generate a vector of normal random numbers at 
       particular mean and variance */
      int row = 1 ;
      int col = 5 ;

      double mean_mu  = 0.56 ;
      double variance = 0.12 ;
      int i ;

      MWNumericArray mw_normalRandVector = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      RandomNumber objMatlab = new RandomNumber() ;
      mw_normalRandVector = (MWNumericArray)objMatlab.myrandn(mw_row, mw_col) ;

      // Convert to double
      double[] db_normalRandVector = (double[])mw_normalRandVector.ToVector(
                                                                 MWArrayComponent.Real) ;
 
      double standard_deviation = Math.Sqrt(variance) ;

      // Print out 
      Console.WriteLine("Normal random numbers with mean = 0.56 and variance = 0.12 :") ;

      for (i=0; i<col; i++)  {
        double aVal = mean_mu + standard_deviation*db_normalRandVector[i] ;
        Console.WriteLine(aVal.ToString() + "\t") ;
      }
  
      Console.WriteLine() ;

      // Free memories
      mw_normalRandVector.Dispose() ;
    }
  
    /* ************************************* */
    public void NormalRandom_matrix()  
    {
      int row = 8 ;
      int col = 5 ;

      RandomNumber objMatlab = new RandomNumber() ;

      MWNumericArray mw_normalRandMatrix = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      // Get matrix of random number from 0 to 1
      mw_normalRandMatrix = (MWNumericArray)objMatlab.myrandn(mw_row, mw_col);

      Console.WriteLine(mw_normalRandMatrix) ;

      // Or convert to double
      double[,] db_normalRandMatrix = (double[,])mw_normalRandMatrix.ToArray(
                                                                MWArrayComponent.Real) ;
      Console.WriteLine() ;
      PrintMatrix(db_normalRandMatrix) ;

      // Free memories  
      mw_normalRandMatrix.Dispose() ;
    }

    /* **************************** */
    public void NormalRandom_matrix2()  
    {
      int row = 8 ;
      int col = 5 ;
      int i, j ;

      double mean_mu  = 0.56 ;
      double variance = 0.12 ;

      double standard_deviation = Math.Sqrt(variance) ;

      RandomNumber objMatlab = new RandomNumber() ;

      MWNumericArray mw_normalRandMatrix = null ;
      MWNumericArray mw_row = new MWNumericArray(row) ;
      MWNumericArray mw_col = new MWNumericArray(col) ;

      // Get matrix of normal random number 
      mw_normalRandMatrix = (MWNumericArray)objMatlab.myrandn(mw_row, mw_col);

      // Convert to double
      double[,] db_normalRandMatrix = (double[,])mw_normalRandMatrix.ToArray(
                                                               MWArrayComponent.Real);

      // Print out 
      Console.WriteLine() ;
      Console.WriteLine(@"The matrix of normal random numbers ");
      Console.WriteLine("at specified mean and variance" );

      for (i=0; i<row; i++)  {
        for (j=0; j<col; j++)  {

          double aVal = mean_mu + standard_deviation*db_normalRandMatrix[i,j] ;
          Console.WriteLine( aVal.ToString() ) ;
        }     
        Console.WriteLine() ;
      }
        
      Console.WriteLine() ;

      // Free memories  
      mw_normalRandMatrix.Dispose() ;
    }

    /* ******************************** */
    public static void PrintVector(double[] vector)
    {
      int i, row;
      row = vector.GetUpperBound(0) - vector.GetLowerBound(0) + 1;

      for (i = 0; i < row; i++)
      {
        Console.Write("{0} \t", vector.GetValue(i).ToString());
        Console.WriteLine();
      }
    }

    /* ******************************** */
    public static void PrintMatrix(double[,] matrix)  
    {  
      int i, j, row, col ;
      row = matrix.GetUpperBound(0) - matrix.GetLowerBound(0) + 1;
      col = matrix.GetUpperBound(1) - matrix.GetLowerBound(1) + 1;

      for (i=0; i<row; i++)  
      {
        for (j=0; j<col; j++) 
        {          
          Console.Write("{0} \t", matrix.GetValue(i,j).ToString() ) ;
        }
        Console.WriteLine() ;
      }

    }

/* ******************************** */
/* ******************************** */
/* ******************************** */  
  }
}
