﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using MathWorks.MATLAB.NET.Utility;
using MathWorks.MATLAB.NET.Arrays ;
using MatrixAdditionNameSpace ;

namespace Example
{
  public partial class Form1 : Form
  {
    public TextBox[,] TextBoxMatrixA ;
    public TextBox[,] TextBoxMatrixB ;
    public TextBox[,] TextBoxMatrixC ;

    public Point StartPoint = new Point() ;
    int TextboxLength ;
    int TextboxHeight ;

    public int NumRow = 3 ;
    public int NumCol = 2 ;

    public double [,] MatrixA_data ;
    public double [,] MatrixB_data ;
    public double [,] MatrixC_data ;
    
    public Form1()
    {
      InitializeComponent();
      InitMatrixes() ;
    }

    /* *************************** */
    public void InitMatrixes() 
    {
      TextBoxMatrixA = (TextBox[,])Array.CreateInstance( typeof(TextBox), NumRow, NumCol ) ;
      TextBoxMatrixB = (TextBox[,])Array.CreateInstance( typeof(TextBox), NumRow, NumCol ) ;
      TextBoxMatrixC = (TextBox[,])Array.CreateInstance( typeof(TextBox), NumRow, NumCol ) ;  

      MatrixA_data = (double[,])Array.CreateInstance( typeof(double), NumRow, NumCol ) ;
      MatrixB_data = (double[,])Array.CreateInstance( typeof(double), NumRow, NumCol ) ;
      MatrixC_data = (double[,])Array.CreateInstance( typeof(double), NumRow, NumCol ) ;
    
      StartPoint.X = 16 ;
      StartPoint.Y = 30 ;

      TextboxLength = 38 ;
      TextboxHeight = 20 ;

      SetTextBoxPositionMatrixA() ;      
      SetTextBoxPositionMatrixB() ;

      labelPlusSign.Location = new System.Drawing.Point(StartPoint.X - 30, 
                                                       (StartPoint.Y + 24) );
      SetTextBoxPositionMatrixC() ;
      labelEqualSign.Location = new System.Drawing.Point(StartPoint.X - 30, 
                                                        (StartPoint.Y + 24) );
    }

    /* *************************** */
    public void SetTextBoxPositionMatrixA()
    {
      for(int i=0; i<NumRow; i++) 
      {
        for(int j=0; j<NumCol; j++)  
        {         
          TextBoxMatrixA[i,j] = new System.Windows.Forms.TextBox();
          TextBoxMatrixA[i,j].Location = new System.Drawing.Point(
                 StartPoint.X + (TextboxLength+2)*j, StartPoint.Y + (TextboxHeight+2)*i );
          TextBoxMatrixA[i,j].Name = "textBox1";
          TextBoxMatrixA[i,j].Size = new System.Drawing.Size(TextboxLength, TextboxHeight);
          TextBoxMatrixA[i,j].TabIndex = i;
          TextBoxMatrixA[i,j].Text = "0.0";
          this.Controls.Add(TextBoxMatrixA[i,j]) ;
        }
      }
    }

    /* *************************** */
    public void SetTextBoxPositionMatrixB()
    {
      StartPoint.X = StartPoint.X + (NumRow*TextboxLength + 10) ;

      for(int i=0; i<NumRow; i++) 
      {
        for(int j=0; j<NumCol; j++)  
        {         
          TextBoxMatrixB[i,j] = new System.Windows.Forms.TextBox();
          TextBoxMatrixB[i,j].Location = new System.Drawing.Point(
                  StartPoint.X + (TextboxLength+2)*j, StartPoint.Y + (TextboxHeight+2)*i );
          TextBoxMatrixB[i,j].Name = "textBox1";
          TextBoxMatrixB[i,j].Size = new System.Drawing.Size(TextboxLength, TextboxHeight);
          TextBoxMatrixB[i,j].TabIndex = i;
          TextBoxMatrixB[i,j].Text = "0.0";
          this.Controls.Add(TextBoxMatrixB[i,j]) ;
        }
      }
    }

    /* *************************** */
    public void SetTextBoxPositionMatrixC()
    {
      StartPoint.X = StartPoint.X + (NumRow*TextboxLength + 10) ;
      for(int i=0; i<NumRow; i++) 
      {
        for(int j=0; j<NumCol; j++)  
        {         
          TextBoxMatrixC[i,j] = new System.Windows.Forms.TextBox();
          TextBoxMatrixC[i,j].Location = new System.Drawing.Point(
                       StartPoint.X + (TextboxLength+2)*j, StartPoint.Y + (TextboxHeight+2)*i );
          TextBoxMatrixC[i,j].Name = "textBox1";
          TextBoxMatrixC[i,j].Size = new System.Drawing.Size(TextboxLength, TextboxHeight);
          TextBoxMatrixC[i,j].TabIndex = i;
          TextBoxMatrixC[i,j].Text = "0.0";
          this.Controls.Add(TextBoxMatrixC[i,j]) ;
        }
      }
    }

    /* *************************** */
    private void buttonMatrixAddition_Click(object sender, EventArgs e)
    {
      ////////// get matrix A /////////////
      try
      {
        GetMatrixA() ;
      }
      catch 
      {
        MessageBox.Show("Data in matrix A is not valid") ;
        return ;
      }
             
      ////////// get matrix B /////////////
      try
      {
        GetMatrixB() ;
      }
      catch 
      {
        MessageBox.Show("Data in matrix B is not valid") ;
        return ;
      }

      this.labelCalcInfo.Text = @"Calculation is in process. Please wait.";
      this.Invalidate() ;
      this.Refresh() ;
        
      /////////////// calculate matrix C ////////////
      GetMatrixC() ;
    }

    /* *************************** */
    public void GetMatrixA()
    {
      int i, j;
      for( i=0; i<NumRow; i++) 
      {
        for( j=0; j<NumCol; j++)  
        {    
          MatrixA_data[i,j] = Convert.ToDouble(TextBoxMatrixA[i,j].Text) ;
        }
      }
    }

    /* *************************** */
    public void GetMatrixB()
    {
      int i, j;
      for( i=0; i<NumRow; i++) 
      {
        for( j=0; j<NumCol; j++)  
        {    
          MatrixB_data[i,j] = Convert.ToDouble(TextBoxMatrixB[i,j].Text) ;
        }
      }
    }

    /* *************************** */
    public void GetMatrixC() 
    {
      // calculate matrix multiplication
      MWNumericArray mw_A = new MWNumericArray(MatrixA_data) ;
      MWNumericArray mw_B = new MWNumericArray(MatrixB_data) ;
      MWNumericArray mw_C = null ;
  
      /* call an implemental function */
      MatrixAddition objMatlab = new MatrixAddition() ;
      mw_C = (MWNumericArray) objMatlab.myplus(mw_A, mw_B) ;      

      /* convert back to C# double  */
      MatrixC_data = (double[,]) mw_C.ToArray(MWArrayComponent.Real) ;

      // display to GUI
      int i, j ;      
      for( i=0; i<NumRow; i++) 
      {
        for( j=0; j<NumCol; j++)  
        {  
          TextBoxMatrixC[i,j].Text = MatrixC_data[i,j].ToString() ;
        }
      }

      labelCalcInfo.Text = @"Calculation has done";

      mw_A.Dispose() ;
      mw_B.Dispose() ;
      mw_C.Dispose() ;
    }

/* *************************** */
/* *************************** */
/* *************************** */
  }
}
